unit CE_Unit;

interface
  uses SysUtils, Controls, Menus, ShellAPI,
    Windows, Dialogs, Graphics, Classes,
    ComCtrls, clipbrd, Messages, ReuUnit,
    ExtCtrls;

  const
    MaxMenuRows = 28;
    MaxMenuCols = 3;
    MAX_FILE_NUM = 255;//MaxMenuRows * MaxMenuCols - 2;
    MAX_REC_NUM = 255;//MaxMenuRows * MaxMenuCols;
    MAX_EXE_NUM = 12;

    EditorName = 'Code Edit';
    EditorNameVer =  EditorName + ' 2.0';
    OptionsFileName = 'ce20.ini';
    FN_DEFAULTS_INI = 'defaults.ini';
    LicenseText = EditorNameVer + ' is freeware!';
    ReadMeFileName = 'readme.txt';
    TempBatFileName = 'temp.bat';
    TempPromptFileName = 'tempprmt.bat';
    FN_TEMP_RTF = 'temp.rtf';
    FN_MAINFORM_BMP = 'codeedit.bmp';
    swLastFile = 0; swNewFile = 1; swNoOpen = 2;
    UNTITLED_PREFIX = 'untitled';

    ValidNums = ['0'..'9', #8, #13, #10];

    //DefaultFilters = 'Any File|*.*|html Files (html)|' + '*.html|htm Files (htm)|' + '*.htm|C++ Files (cpp)|*.cpp|C Files (c)|*.c|Header Files (h)|*.h|Java Files (java)|*.java|Batch Files (bat)|*.bat|Assembly Files (asm)|*.asm|Pascal Files (pas)|*.pas|Basic Files (bas)|*.bas|Text Files (txt)|*.txt|m3u Files (m3u)|*.m3u';
    RegNum = '3141592654';

    //lcl: link color
    lclOver = clRed;
    lclDefault = clBlue;

    PR_TOKEN = '$';
    PR_REMOVE_EXTENSION = 'x';
    PR_EXTRACT_PATH = 'p';
    PR_REMOVE_PATH = 'f';
    PR_DOS_NAME = 'd';
    PR_DIRVE_LETTER = 'v';
    PR_PROMPT = '%p%';
    ValidFilePRs = [PR_REMOVE_EXTENSION, PR_EXTRACT_PATH, PR_REMOVE_PATH, PR_DOS_NAME, PR_DIRVE_LETTER];

    //PN = panel
    PN_POS = 0;
    PN_MODIFIED = 1;
    PN_INSERT = 2;
    PN_CLIP = 3;

  type
    TColorScheme = (csWindows, csPascal, csCustom);
    TKeyMap = (kmWindows, kmPascal);
    TCanOpen = (coNotExists, coTooBig, coMaxFiles, coOptionsFile, coTrue);
    TBackupExt = (beBak, beTilde);
    TCMMode = (cmCopy, cmMove); // for CMUnit
    TFileAttr = SmallInt;

    TFileNum = byte;

    TProg = record
      SaveOrBackupIndex : byte;
      Prog, Params : string;
      BatFile : TStringList;
    end;

    TExeOpt = object
      public
        SaveOrBackupIndex : byte;
        SourceExts : TStringList;
        ExeExt : TFileName;
        cd, ParamsPrompt : boolean;

        procedure Assign (Item : TExeOpt);
        constructor Create;
        destructor Free;
    end;

    TExeOpts = object
      public
        Items : array[0..MAX_EXE_NUM - 1] of ^TExeOpt;

        procedure Create;
        procedure Free;
        function Add (Item : TExeOpt) : boolean;
        function Delete (index : Cardinal) : boolean;
        function Insert (Item : TExeOpt; index : Cardinal) : boolean;
        procedure Update (Item : TExeOpt; index : Cardinal);
        function GetCount : Cardinal;
        function IndexOfSource (ext : TFileName) : integer;

      private
        Count : Cardinal;
    end;

    TFile = object
    public
      FileName : TFileName;
      Text : string;
      ReadOnly, Modified : boolean;
      {FirstVisibleLine,} SelStart, SelLength : cardinal;
      VertScrollPos, HorzScrollPos : integer;

      procedure Save;
      procedure Assign (cFile : TFile);
    end;

    TSelectionStatus = object
    private
      FStatus : boolean;
      FStart, FStop : integer;

    public
      property Status : boolean read FStatus;
      property Start : integer read FStart;
      property Stop : integer read FStop;

      procedure SetWithLines(st, sp : integer);
      procedure Clear;
    end;

    TStringListPtr = ^TStringList;

    TStringListList = class(TList)
    private
      procedure SetSL(i : integer; sl : TStringListPtr);
      function GetSL(i : integer) : TStringListPtr;

    public
      property Items[i : integer] : TStringListPtr
        read GetSL
        write SetSL; default;

      procedure LoadFromFile(var FileVar : TextFile);
      procedure SaveToFile(var FileVar : TextFile);
    end;

    TTextDefaults = object
    private
      Exts : TStringList;
      Texts : TStringListList;

    public
      function GetText(const ext : TFileName) : string;
      procedure SetText(const ext : TFileName; const s : string);
      procedure LoadFromFile(var FileVar : TextFile);
      procedure SaveToFile(var FileVar : TextFile);
      constructor Create;
      destructor Free;
    end;

  IntegerPtr = ^integer;
  function newInteger(i : integer) : IntegerPtr;
  function StringToLabel (const s : string) : string;
  function AssignLastCall (b : boolean) : boolean;
  procedure ValidateKey (var Key : char);
  function GetShowCmd (const fName : TFileName) : integer;
  function ShellExe (const s : string) : integer;
  function IsReadOnly (const fName : TFileName) : boolean;
  function CapToStr (const s : TCaption) : string;
  function StrToCap (const s : string; i : byte) : TCaption;
  function MenuItemToStringList (const MenuItem : TMenuItem) : TStringList;
  function FindCapMenuIndex (const MenuItem : TMenuItem; const s : string) : integer;
  function ReadOnlyFile (const s : string; b : boolean) : boolean;
  function Int (const s : string) : integer;
  function IntPower(base, exp : integer) : integer;
  function IsIdenifier(const name : string) : boolean;
  function IsValidSymbol(const name : string) : boolean;
  function LwrChr (w : word) : char;
  function FileToStr (const FileName : TFileName): string;
  function StrRev (const s1 : string): string;
  procedure AddBreak (var Menu : TPopupMenu);
  function IsUntitled (const fName : TFileName) : boolean;
  function Execute (const s : string) : integer;
  function ColorToIndex (color : TColor): byte;
  function Timer : real;
  function FindOptionsToSearchTypes (fo : TFindOptions) : TSearchTypes;
  procedure MakeTempBatFile (const TempName, fName : TFileName; const cmd : string; AddPause : boolean);
  procedure DeployBatFile (const cmd : string);
  procedure FixColor;
  procedure CopyMenuItem (var MI1 : TMenuItem; const MI2 : TMenuItem);
  function GetItem (const s : string; index : integer; Separator : Char) : string;

  var SourceDir : string;

implementation

uses MainUnit;

// ****** OBJECT METHODS ******
procedure TFile.Save;
var FileVar : TextFile;
begin
  if not Modified then exit;

  if IsReadOnly (FileName) then begin
    raise EWriteError.Create ('This is a readonly file.  If you want to save this file, use a different name.');
    exit;
  end;

  try
    AssignFile (FileVar, FileName);
    Rewrite (FileVar);
      Write (FileVar, Text);
    CloseFile (FileVar);
    Modified := false;
  except
    CloseFile (FileVar);
    raise;
  end;
end;

procedure TFile.Assign (cFile : TFile);
begin
  FileName := cFile.FileName;
  Text := cFile.Text;
  ReadOnly := cFile.ReadOnly;
  Modified := cFile.Modified;
  SelStart := cFile.SelStart;
  SelLength := cFile.SelLength;
  //FirstVisibleLine := cFile.FirstVisibleLine;
  VertScrollPos := cFile.VertScrollPos;
  HorzScrollPos := cFile.HorzScrollPos;
end;

procedure TExeOpt.Assign (Item : TExeOpt);
begin
  SaveOrBackupIndex := Item.SaveOrBackupIndex;
  SourceExts.CommaText := Item.SourceExts.CommaText;
  ExeExt := Item.ExeExt;
  cd := Item.cd;
  ParamsPrompt := Item.ParamsPrompt;
end;

constructor TExeOpt.Create;
begin
  SourceExts := TStringList.Create;
end;

destructor TExeOpt.Free;
begin
  SourceExts.Free;
end;

function TExeOpts.Add (Item : TExeOpt) : boolean;
begin
  result := Count < MAX_EXE_NUM;
  if not result then exit;

  new (Items[Count]);
  Items[Count]^.Create;
  Items[Count]^.Assign(Item);
  inc (Count);
end;

function TExeOpts.Delete (index : Cardinal) : boolean;
var i : cardinal;
begin
  result := index < Count;
  if not result then exit;

  if index <= Count - 2 then
    for i := index to Count - 2 do
      Items[i].Assign (Items[i + 1]^);

  dec (Count);
  Items[Count]^.Free;
  Dispose (Items[Count]);
end;

function TExeOpts.Insert (Item : TExeOpt; index : Cardinal) : boolean;
var i : Cardinal;
begin
  result := index <= Count;
  if not result then exit;

  if index = Count then begin
    Add (Item);
    exit;
  end;

  Add (Items[Count - 1]^);
  for i := Count - 2 downto index + 1 do
    Items[i].Assign (Items[i - 1]^);

  Items[index].Assign (Item);
end;

procedure TExeOpts.Update (Item : TExeOpt; index : Cardinal);
begin
  if index >= Count then
    Add (Item)
  else
    Items[index].Assign (Item);
end;

function TExeOpts.IndexOfSource (ext : TFileName) : integer;
begin
  if pos('.', ext) = 1 then system.delete(ext, 1, 1);

  if Count > 0 then
    for result := 0 to Count - 1 do
      if Items[result]^.SourceExts.IndexOf(ext) <> -1 then exit;

  result := -1;
end;

procedure TExeOpts.Create;
begin
  Count := 0;
end;

procedure TExeOpts.Free;
var i : cardinal;
begin
  if Count = 0 then exit;

  for i := 0 to Count - 1 do
    Dispose (Items[i]);
end;

function TExeOpts.GetCount : Cardinal;
begin
  result := Count;
end;

procedure TSelectionStatus.SetWithLines(st, sp : integer);
begin
  FStatus := true;
  FStart := st;
  FStop := sp;
end;

procedure TSelectionStatus.Clear;
begin
  FStatus := false;
end;

procedure TStringListList.SetSL(i : integer; sl : TStringListPtr);
begin
  List[i] := sl;
end;

function TStringListList.GetSL(i : integer) : TStringListPtr;
begin
  result := TStringListPtr(List[i]);
end;

procedure TStringListList.LoadFromFile(var FileVar : TextFile);
var
  i : integer;
  sl : TStringListPtr;
begin
  ReadLn(FileVar, i);
  Clear;

  while Count < i do begin
    new(sl);
    sl^ := TStringList.Create;
    LoadStringListFromFile(sl^, FileVar);
    Add(sl);
  end;
end;

procedure TStringListList.SaveToFile(var FileVar : TextFile);
var i : integer;
begin
  WriteLn(FileVar, Count);
  if Count > 0 then
    for i := 0 to Count - 1 do
      SaveStringListToFile(Items[i]^, FileVar);
end;

function TTextDefaults.GetText(const ext : TFileName) : string;
var i : integer;
begin
  i := Exts.IndexOf(ext);
  if i = -1 then
    result := ''
  else
    result := Texts[i].Text;
end;

procedure TTextDefaults.SetText(const ext : TFileName; const s : string);
var
  i : integer;
  sl : TStringListPtr;
begin
  i := Exts.IndexOf(ext);
  if i = -1 then begin
    if s = '' then exit;
    Exts.Add(ext);
    new(sl);
    sl^ := TStringList.Create;
    sl^.Text := s;
    Texts.Add(sl);
  end else
    if s = '' then begin
      Texts.Delete(i);
      Exts.Delete(i);
    end else
      Texts[i]^.Text := s;
end;

procedure TTextDefaults.LoadFromFile(var FileVar : TextFile);
begin
  LoadStringListFromFile(Exts, FileVar);
  Texts.LoadFromFile(FileVar);
end;

procedure TTextDefaults.SaveToFile(var FileVar : TextFile);
begin
  SaveStringListToFile(Exts, FileVar);
  Texts.SaveToFile(FileVar);
end;

constructor TTextDefaults.Create;
begin
  Exts := TStringList.Create;
  Texts := TStringListList.Create;
end;

destructor TTextDefaults.Free;
begin
  Exts.Free;
  Texts.Free;
end;

// ****** PROCEDURES AND FUNCTIONS ******
function newInteger(i : integer) : IntegerPtr;
begin
  new(result);
  result^ := i;
end;

{this function changes & to && and is used for situations
 where a string that has &'s will be assigned to something
 where &'s are interpreted to underscore the next
 character; &&'s in those cases are interpreted as &'s}
function StringToLabel (const s : string) : string;
const TempStr = '?#*#?';
begin
  result := s;
  ReplaceAll ('&', TempStr, result);
  ReplaceAll (TempStr, '&&', result);
end;

{returns the value with which AssignLastCall was called
 last time and uses b to return upon next call}
function AssignLastCall (b : boolean) : boolean;
const LastB : boolean = true;
begin
  result := LastB;
  LastB := b;
end;

procedure ValidateKey (var Key : char);
begin
  if not (Key in ValidNums) then Key := #0;
end;

{specifices nShowCmd for ShellExecute;
 returns SW_SHOWDEFAULT for executable files
 and 0 otherwise}
function GetShowCmd (const fName : TFileName) : integer;
var fNameExt : TFileName;
begin
  fNameExt := ExtractFileExt (fName);
  if StrCmpi (fNameExt, '.exe')
  or StrCmpi (fNameExt, '.com')
  or StrCmpi (fNameExt, '.bat') then
    result := SW_SHOWDEFAULT
  else
    result := 0;
end;

function ShellExe (const s : string) : integer;
begin
  result := ShellExecute (0, 'open', PChar (s), Nil, Nil, GetShowCmd (s));
  case result of
    SE_ERR_FNF : ShowMsgDlg ('File "' + s + '" not found', mtError);
    SE_ERR_PNF : ShowMsgDlg ('Path "' + ExtractFilePath (s) + '" not found', mtError);
    SE_ERR_ACCESSDENIED : ShowMsgDlg ('Access to "' + s + '" denied', mtError);
    SE_ERR_OOM : ShowMsgDlg ('Out of memory', mtError);
    SE_ERR_DLLNOTFOUND : ShowMsgDlg ('DLL not found', mtError);
  end;
end;

function IsReadOnly (const fName : TFileName) : boolean;
begin
  result := FileExists (fName) and
    (FileGetAttr (fName) and faReadOnly = faReadOnly);
end;

{if s = '&1 file.txt' then the function will
 return 'file.txt'}
function CapToStr (const s : TCaption) : string;
begin
  if (s = '') or (s[1] <> '&') then begin
    result := s;
    exit;
  end;

  result := copy (s, 4, length (s) - 3);
end;

{if i = 1 and s = 'file.txt' then funciton
 will return '&1 file.txt'}
function StrToCap (const s : string; i : byte) : TCaption;
begin
  result := s;
  if i > 9 then exit;
  result := '&' + chr (i + ord ('0')) + ' ' + result;
end;

function MenuItemToStringList (const MenuItem : TMenuItem) : TStringList;
var i : cardinal;
begin
  Result := TStringList.Create;
  if MenuItem.Count = 0 then exit;
  for i := 0 to MenuItem.Count - 1 do
    Result.Add (CapToStr (MenuItem.Items[i].Caption));
end;

function FindCapMenuIndex (const MenuItem : TMenuItem; const s : string) : integer;
var i : integer;
begin
  if MenuItem.Count = 0 then begin
    result := -1;
    exit;
  end;

  for i := 0 to MenuItem.Count - 1 do
    if StrCmpi (s, CapToStr (MenuItem[i].Caption)) then begin
      result := i;
      exit;
    end;

   result := -1;
end;

function ReadOnlyFile (const s : string; b : boolean) : boolean;
var attr : word;
begin
  result := FileExists (s);
  if not result then exit;

  attr := FileGetAttr (s);
  if b then
    attr := attr or faReadOnly
  else
    attr := (attr and faSysFile)
      or (attr and faHidden)
      or (attr and faArchive)
      or (attr and faVolumeID)
      or (attr and faDirectory);

  FileSetAttr (s, attr);
end;

function Int (const s : string) : integer;
var
  len, i : cardinal;
begin
  len := length (s);
  if len = 0 then begin
    result := 0;
    exit;
  end;

  for i := 1 to len do
    if not (s[i] in ['0'..'9']) then begin
      result := 0;
      exit;
    end;

  result := StrToInt (s);
end;

function IntPower(base, exp : integer) : integer;
begin
  assert(exp >= 0);
  if exp = 0 then
    result := 1
  else
    result := base * IntPower(base, exp - 1);
end;

{returns true iff name is non-empty, starts with a letter
 or underscore, and contains only letters, numbers, and
 underscores}
function IsIdenifier(const name : string) : boolean;
var i : integer;
begin
  i := length(name);
  if i = 0 then begin
    result := false;
    exit;
  end;

  while i > 1 do begin
    if not IsAlphaNum(name[i]) and not (name[i] = '_') then begin
      result := false;
      exit;
    end;
    dec(i);
  end;

  result := IsAlpha(name[1]) or (name[1] = '_');
end;

function IsValidSymbol(const name : string) : boolean;
begin
  result := (length(name) > 0) and
    not IsDigit(name[1]) and
    not IsIdenifier(name);
end;

function LwrChr (w : word) : char;
begin
  if w in [65..90] then
    w := w + 32;

  result := chr (w);
end;

function FileToStr (const FileName : TFileName): string;
var
  FileVar : TextFile;
  InStr : string;
begin
  if not FileExistsEx (FileName, true) then exit;

  AssignFile (FileVar, FileName);
  ReSet (FileVar);
    ReadLn (FileVar, result);
    while not eof (FileVar) do begin
      ReadLn (FileVar, InStr);
      result := result + #13 + #10 + InStr;
    end;
  CloseFile (FileVar);
end;

function StrRev (const s1 : string) : string;
var
  len, i : cardinal;
begin
  len := length (s1);
  for i := 1 to len do
    result := result + s1[len - i + 1];

end;

procedure AddBreak (var Menu : TPopupMenu);
var MenuItem : TMenuItem;
begin
  MenuItem := TMenuItem.Create (Menu);
  MenuItem.Caption := '-';
  Menu.Items.Add (MenuItem);
end;

function IsUntitled (const fName : TFileName) : boolean;
//var i : byte;
begin
  result := UNTITLED_PREFIX = copy(fName, 1, length(UNTITLED_PREFIX));

  {is = position of last number--just before
   extension if there is one, or at end of
   fName otherwise}
  {i := Pos ('.', fName);
  if i = 0 then
    i := length (fName)
  else
    dec (i);

  if ('Untitled' = copy (fName, 1, 8))
  and (StrToInt (Copy (fName, 9, i - 8)) > 0)  then
    result := true
  else
    result := false;}
end;

function Execute (const s : string) : integer;
begin
  result := WinExec (PChar (s), SW_SHOWDEFAULT);
  case result of
    0 : ShowMsgDlg ('The system is out of memory or resources.', mtError);
    ERROR_BAD_FORMAT : ShowMsgDlg ('The .EXE file is invalid (non-Win32 .EXE or error in .EXE image).', mtError);
    ERROR_FILE_NOT_FOUND : ShowMsgDlg ('File "' +  s + '" not found.', mtError);
    ERROR_PATH_NOT_FOUND : ShowMsgDlg ('Path "' + ExtractFilePath (s) + '" not found.', mtError);
  end;
end;

function ColorToIndex (color : TColor): byte;
begin
  {used only in EnvironmentUnit}
  case Color of
    clBlack : result := 0;
    clMaroon : result := 1;
    clGreen : result := 2;
    clOlive : result := 3;
    clNavy : result := 4;
    clPurple : result := 5;
    clTeal : result := 6;
    clSilver : result := 7;
    clGray : result := 8;
    clRed : result := 9;
    clLime : result := 10;
    clYellow : result := 11;
    clBlue : result := 12;
    clFuchsia : result := 13;
    clAqua : result := 14;
    clWhite : result := 15;
    else result := 255;
  end;
end;

function Timer : real;
var Hour, Min, Sec, MSec : word;
begin
  DecodeTime (Time, Hour, Min, Sec, MSec);
  result := Min * 60 + Sec + MSec / 1000;
end;

function FindOptionsToSearchTypes (fo : TFindOptions) : TSearchTypes;
begin
  if frMatchCase in fo then
    result := result + [stMatchCase];

  if frWholeWord in fo then
    result := result + [stWholeWord];
end;

{makes a batch file w/ name = the const
 TempBatFileName which
 1. executes command.com
 2. changes to fName's drive and direcotry
 3. executes parameter cmd

 it is used for running DOS prompts or commands}
procedure MakeTempBatFile (const TempName, fName : TFileName; const cmd : string; AddPause : boolean);
var FileVar : TextFile;
begin
  AssignFile (FileVar, SourceDir + TempName);
  rewrite (FileVar);
    writeln (FileVar, '@echo off');

    {if statements see if fName starts with
     a valid drive letter}
    if pos (':', fName) = 2 then
      if DiskFree (CharToDriveNum (fName[1])) <> -1 then
      begin
        //change drive
        writeln (FileVar, copy (fName, 1, 2));
        //change directory
        writeln (FileVar, 'cd ' + ExtractFilePath (fName));
      end;

    writeln (FileVar, cmd);
    if AddPause then writeln (FileVar, 'pause');
  CloseFile (FileVar);
end;

{if TempBatFileName exists then cmd is added to the end
 of it, otherwise it is created and cmd is writen to it}
procedure DeployBatFile (const cmd : string);
var FileVar : TextFile;
begin
  AssignFile (FileVar, SourceDir + TempBatFileName);
  Rewrite (FileVar);
    writeln (FileVar, cmd);
  CloseFile (FileVar);

  Execute (SourceDir + TempBatFileName);
end;

{makes text in the clipboard lose formating
 info. such as coloring}
procedure FixColor;
begin
  Clipboard.AsText := Clipboard.AsText;
end;

procedure CopyMenuItem (var MI1 : TMenuItem; const MI2 : TMenuItem);
var
  i : cardinal;
  MI : Pointer;
begin
  MI1.Caption := MI2.Caption;
  MI1.ShortCut := MI2.ShortCut;
  MI1.OnClick := MI2.OnClick;

  if MI2.Count = 0 then exit;

  while MI1.Count > 0 do
    MI1.Delete(0);

  for i := 0 to MI2.Count - 1 do begin
    GetMem(MI, sizeof(MI2.Items[i]));
    if MI2.Items[i] is TGroupMenuItem then
      TGroupMenuItem(MI^) := TGroupMenuItem.Create(MI2)
    else
      TMenuItem(MI^) := TMenuItem.Create (MI1);

    CopyMenuItem (TMenuItem(MI^), MI2.Items[i]);
    MI1.Add (TMenuItem(MI^));
  end;
end;

{if Separator = '|' and s = 'item1|item2|item3' and
 index = 1, then result = 'item2'}
function GetItem (const s : string; index : integer; Separator : Char) : string;
var i, len : cardinal;
begin
  result := '';
  len := length (s);
  if len = 0 then exit;

  i := 1;
  while i <= len do begin
    if s[i] = Separator then begin
      inc (i);
      if i > len then exit;
      dec (index);
    end;

    if index = 0 then result := result + s[i];
    if index < 0 then exit;

    inc (i);
  end;
end;

end.
