unit CMUnit;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Buttons, FileCtrl,
  CE_Unit, ReuUnit, Grids, Outline, DirOutln, ExtCtrls;

type
  TCMForm = class(TForm)
    Label1: TLabel;
    FolderCB: TComboBox;
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    DirLB: TDirectoryListBox;
    DriveCB: TDriveComboBox;
    Bevel1: TBevel;
    procedure FormShow(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure DriveCBClick(Sender: TObject);
    procedure DriveCBDropDown(Sender: TObject);
    procedure DriveCBKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormHide(Sender: TObject);
    procedure DirLBChange(Sender: TObject);
    procedure DirLBEnter(Sender: TObject);
    procedure DirLBKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    LastDriveIndex : SmallInt;
    function ValidateDrive : boolean;
  public
    { Public declarations }
    mode : TCMMode;
  end;

var
  CMForm: TCMForm;

implementation

uses MainUnit;

{$R *.DFM}

{returns false if invalid drive was selected then cancel
 was chosen between retry and cancel}
function TCMForm.ValidateDrive : boolean;
begin
  if DiskSize (Ord (DriveCB.Text[1]) - Ord ('A') + 1) = - 1 then begin
    if MessageDlg ('Drive "' + DriveCB.Text + '" is not ready!',
                   mtError, [mbRetry, mbCancel], 0) = mrRetry then
      result := ValidateDrive
    else begin
      DriveCB.ItemIndex := LastDriveIndex;
      result := false;
    end;
  end else result := true;
end;

procedure TCMForm.FormShow(Sender: TObject);
begin
  Application.ShowHint := AssignLastCall (Application.ShowHint);
  if mode = cmCopy then begin
    Caption := 'Copy';
    Label1.Caption := 'Copy &File "' + MainForm.FileName + '" to:';
  end else begin
    Caption := 'Move';
    Label1.Caption := 'Move &File "' + MainForm.FileName + '" to:';
  end;
  CropLabel (Label1);
end;

procedure TCMForm.OKBtnClick(Sender: TObject);
var ToFile : TFileName;
begin
  if not ValidateDrive then begin
    ModalResult := mrNone;
    exit;
  end;

  FolderCB.Text := ExpandFileName (FolderCB.Text);

  if FolderCB.GetTextLen = 0 then
    FolderCB.Text := ExpandFileName ('\')
  else if FolderCB.Text[FolderCB.GetTextLen] <> '\' then
    FolderCB.Text := FolderCB.Text + '\';

  if not DirectoryExists (FolderCB.Text) then begin
    if MessageDlg ('Directory "' + FolderCB.Text + '" does not exist! Do you want to create it?',
                   mtConfirmation, [mbYes, mbNo], 0) = mrNo then begin
      ModalResult := mrNone;
      exit;
    end;

    if not CreateDir (FolderCB.Text) then begin
      ShowMsgDlg ('Could not create directory "' + FolderCB.Text + '"', mtError);
      ModalResult := mrNone;
      exit;
    end;
  end;

  ToFile := FolderCB.Text + ExtractFileName (MainForm.FileName);
  if FileExistsEx (ToFile, true) then
    if MessageDlg ('File "' + ToFile + '" eixsts.  Overwright?',
                mtConfirmation, [mbYes, mbNo], 0) = mrNo then begin
      ModalResult := mrNone;
      exit;
    end;

  if mode = cmCopy then begin
    if not CopyFile (PChar (MainForm.FileName), PChar (ToFile), FALSE) then begin
      ShowMsgDlg ('File "' + MainForm.FileName + '" could not be copied', mtError);
      ModalResult := mrNone;
      exit;
    end;
  end else begin
    if not MoveFile (PChar (MainForm.FileName), PChar (ToFile)) then begin
      ShowMsgDlg ('File "' + MainForm.FileName + '" could not be moved', mtError);
      ModalResult := mrNone;
      exit;
    end;
    MainForm.ChangeGroupFile (MainForm.FileName, ToFile);
    MainForm.ChangeFavFile (MainForm.FileName, ToFile);
    MainForm.ChangeRecentFile (MainForm.FileName, ToFile);
    MainForm.FileName := ToFile;
  end;

  if FolderCB.Items.IndexOf (FolderCB.Text) < 0 then
    FolderCB.Items.Add (FolderCB.Text);
end;

procedure TCMForm.DriveCBClick(Sender: TObject);
begin
  ValidateDrive;
end;

procedure TCMForm.DriveCBDropDown(Sender: TObject);
begin
  LastDriveIndex := DriveCB.ItemIndex;
end;

procedure TCMForm.DriveCBKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  LastDriveIndex := DriveCB.ItemIndex;
end;

procedure TCMForm.FormHide(Sender: TObject);
begin
  Application.ShowHint := AssignLastCall (true);
end;

procedure TCMForm.DirLBChange(Sender: TObject);
begin
  FolderCB.Text := DirLB.Directory;
end;

procedure TCMForm.DirLBEnter(Sender: TObject);
begin
  FolderCB.Text := DirLB.Directory;
end;

procedure TCMForm.DirLBKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_RIGHT then DirLB.OpenCurrent;
end;

end.

