unit EnvironmentUnit;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  ColorGrd, StdCtrls, ExtCtrls, Buttons,
  ComCtrls, Spin, Grids,
  CE_Unit, ReuUnit, Menus;

type
  TEnvironmentForm = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Bevel1: TBevel;
    KeystrokeRG: TRadioGroup;
    StartWithRG: TRadioGroup;
    BackupExtRG: TRadioGroup;
    GroupBox4: TGroupBox;
    AutosaveCB: TCheckBox;
    IntervalLab: TLabel;
    IntervalSE: TSpinEdit;
    HelpBtn: TBitBtn;
    GroupBox5: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    ShowHintsCB: TCheckBox;
    HintElapseEdit: TEdit;
    HintDurationEdit: TEdit;
    TabsGB: TGroupBox;
    TabSizeSE: TSpinEdit;
    TabSizeLab: TLabel;
    GroupBox1: TGroupBox;
    PersistentGroupsCB: TCheckBox;
    ConfirmPromptCB: TCheckBox;
    AutoIndentCB: TCheckBox;
    UseSpacesCB: TCheckBox;
    GroupBox2: TGroupBox;
    WindowsDefaultRB: TRadioButton;
    PascalRB: TRadioButton;
    CustomRB: TRadioButton;
    ColorGrid1: TColorGrid;
    procedure OKBtnClick(Sender: TObject);
    procedure ColorGrid1Change(Sender: TObject);
    procedure AutosaveCBClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure IntervalSEKeyPress(Sender: TObject; var Key: Char);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    function GetColorScheme : TColorScheme;
    procedure SetColorScheme(cs : TColorScheme);

    property ColorScheme : TColorScheme
      read GetColorScheme
      write SetColorScheme;

    procedure ChangeAuto;
  public
    { Public declarations }
  end;

var
  EnvironmentForm: TEnvironmentForm;

implementation

uses MainUnit, OrderUnit;

{$R *.DFM}

function TEnvironmentForm.GetColorScheme : TColorScheme;
begin
  if WindowsDefaultRB.Checked then result := csWindows
  else if PascalRB.Checked then result := csPascal
  else result := csCustom
end;

procedure TEnvironmentForm.SetColorScheme(cs : TColorScheme);
begin
  case cs of
    csWindows : WindowsDefaultRB.Checked := true;
    csPascal : PascalRB.Checked := true;
    csCustom : CustomRB.Checked := true;
  end;
end;

procedure TEnvironmentForm.ChangeAuto;
begin
  IntervalSE.Enabled := AutoSaveCB.Checked;
  IntervalLab.Enabled := AutoSaveCB.Checked;
end;

procedure TEnvironmentForm.OKBtnClick(Sender: TObject);
begin
  with MainForm do begin
    SaveAllEditAttributes;

    if KeystrokeRG.ItemIndex = 0 then
      ChangeKeyMap (kmWindows)
    else
      ChangeKeyMap (kmPascal);

    CustomBack := ColorGrid1.BackGroundColor;
    CustomFore := ColorGrid1.ForeGroundColor;
    AssignColorScheme (Self.ColorScheme);

    Timer1.Interval := IntervalSE.Value * 60000;
    Timer1.Enabled := AutoSaveCB.Checked;

    StartWith := StartWithRG.ItemIndex;

    BackupExt := IntToBackupExt (BackupExtRG.ItemIndex);

    PersistentGroups := PersistentGroupsCB.Checked;
    ConfirmPrompt := ConfirmPromptCB.Checked;
    AutoIndent := AutoIndentCB.Checked;

    TabUse.Load (UseSpacesCB, TabSizeSE);

    LoadAllEditAttributes;
  end;//with MainForm

  with Application do begin
    ShowHint := ShowHintsCB.Checked;
    HintPause := Int (HintElapseEdit.Text );
    HintHidePause := Int (HintDurationEdit.Text);
  end;
end;

procedure TEnvironmentForm.ColorGrid1Change(Sender: TObject);
begin
  ColorScheme := csCustom;
end;

procedure TEnvironmentForm.AutoSaveCBClick(Sender: TObject);
begin
  ChangeAuto;
end;

procedure TEnvironmentForm.FormShow(Sender: TObject);
begin
  with MainForm do begin
    ColorGrid1.ForeGroundIndex := ColorToIndex (CustomFore);
    ColorGrid1.BackGroundIndex := ColorToIndex (CustomBack);

    Self.ColorScheme := ColorScheme;

    KeystrokeRG.ItemIndex := Ord (KeyMap);
    AutoSaveCB.Checked := Timer1.Enabled;
    ChangeAuto;
    IntervalSE.Value := Round (Timer1.Interval div 60000);
    StartWithRG.ItemIndex := StartWith;

    BackupExtRG.ItemIndex := Ord (BackupExt);

    PersistentGroupsCB.Checked := PersistentGroups;
    ConfirmPromptCB.Checked := ConfirmPrompt;
    AutoIndentCB.Checked := AutoIndent;

    TabUse.Store (UseSpacesCB, TabSizeSE);
  end;

  with Application do begin
    ShowHintsCB.Checked := ShowHint;
    HintElapseEdit.Text := IntToStr (HintPause);
    HintDurationEdit.Text := IntToStr (HintHidePause);
  end;
end;

procedure TEnvironmentForm.IntervalSEKeyPress(Sender: TObject;
  var Key: Char);
begin
  ValidateKey (Key);
end;

procedure TEnvironmentForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_F1 then HelpBtn.Click;
end;


end.

