unit ExecuteOptionsUnit;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons, ComCtrls,
  CE_Unit, ReuUnit;

type
  TExeOptForm = class(TForm)
    Bevel1: TBevel;
    OnExecGB: TGroupBox;
    ParamsPromptCB: TCheckBox;
    cdCB: TCheckBox;
    SaveOrBackupRG: TRadioGroup;
    HelpBtn: TBitBtn;
    ExtGB: TGroupBox;
    Label3: TLabel;
    Label4: TLabel;
    SourceEdit: TEdit;
    ExeEdit: TEdit;
    RecNumGB: TGroupBox;
    RecNumEdit: TEdit;
    RecNumSB: TScrollBar;
    NextBtn: TBitBtn;
    PreBtn: TBitBtn;
    CloseBtn: TBitBtn;
    NewBtn: TBitBtn;
    DeleteBtn: TBitBtn;
    ReloadBtn: TBitBtn;
    SaveBtn: TBitBtn;
    ClearBtn: TBitBtn;
    GroupBox1: TGroupBox;
    HotKey: THotKey;
    procedure FormShow(Sender: TObject);
    procedure ClearBtnClick(Sender: TObject);
    procedure SaveBtnClick(Sender: TObject);
    procedure NewBtnClick(Sender: TObject);
    procedure ReloadBtnClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure NextBtnClick(Sender: TObject);
    procedure PreBtnClick(Sender: TObject);
    procedure RecNumSBScroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
    procedure RecNumEditKeyPress(Sender: TObject; var Key: Char);
    procedure SourceEditChange(Sender: TObject);
    procedure ExeEditChange(Sender: TObject);
    procedure ParamsPromptCBClick(Sender: TObject);
    procedure cdCBClick(Sender: TObject);
    procedure SaveOrBackupRGClick(Sender: TObject);
    procedure ExeEditKeyPress(Sender: TObject; var Key: Char);
    procedure FormHide(Sender: TObject);
    procedure RecNumEditExit(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    CurRec, NumOfRecs : Cardinal;
    modified : boolean;

    procedure ChangeCurRec (num : Cardinal);
    procedure EnableButtons;
    function GetRecAsExeOpt : TExeOpt;
    function Suffice : boolean;
    function IsSaved : boolean;
    procedure SetModified;
    function PromptToChangeRec : boolean;
    function PromptToSave : boolean;
    function SaveWithDup : boolean;
    procedure ShowCurRec;
    procedure SetNumOfRecs (num : Cardinal);
  public
    { Public declarations }
  end;

var
  ExeOptForm: TExeOptForm;

implementation

uses MainUnit;

{$R *.DFM}

procedure TExeOptForm.ChangeCurRec (num : Cardinal);
begin
  if not PromptToSave then
    if not PromptToChangeRec then exit;

  CurRec := num;
  ShowCurRec;
end;

procedure TExeOptForm.EnableButtons;
begin
  DeleteBtn.Enabled := IsSaved;
  ReloadBtn.Enabled := modified and IsSaved;
  SaveBtn.Enabled := modified and Suffice;
  NewBtn.Enabled := CurRec < MAX_EXE_NUM;
  PreBtn.Enabled := CurRec > 1;
  NextBtn.Enabled := CurRec < NumOfRecs;
end;

function TExeOptForm.GetRecAsExeOpt : TExeOpt;
begin
  with result do begin
    Create;
    SaveOrBackupIndex := SaveOrBackupRG.ItemIndex;
    SourceExts.CommaText := SourceEdit.Text;
    ExeExt := ExeEdit.Text;
    if pos('.', ExeExt) = 1 then delete(ExeExt, 1, 1);
    cd := cdCB.Checked;
    ParamsPrompt := ParamsPromptCB.Checked;
  end;
end;

function TExeOptForm.Suffice : boolean;
begin
  result := (SourceEdit.Text <> '') and (ExeEdit.Text <> '');
end;

procedure TExeOptForm.SetModified;
begin
  modified := true;
  EnableButtons;
end;

function TExeOptForm.IsSaved : boolean;
begin
  result := CurRec <= MainForm.ExeOpts.GetCount;
end;

function TExeOptForm.PromptToChangeRec : boolean;
begin
  result := MessageDlg ('Do you still wish to change the current record number?',
                 mtConfirmation, [mbYes, mbNo], 0) = mrYes;
end;

function TExeOptForm.PromptToSave : boolean;
begin
  result := true;

  if modified then
    if Suffice then begin
      if MessageDlg ('The current record has been modifed.  Do you wish to SAVE it?',
                     mtConfirmation, [mbYes, mbNo], 0) = mrYes then
        SaveBtn.Click
      else result := false;
    end else if IsSaved then
      if MessageDlg ('The current record has been modifed and now conatins insufficient data.  Do you wish to DELETE it?',
                     mtConfirmation, [mbYes, mbNo], 0) = mrYes then
        DeleteBtn.Click
      else result := false;
end;

{returns true if an extension specified in SourceEdit
 either does not match a SourceExt in a previous record,
 or if it does and the user responds Yes to a warning}
function TExeOptForm.SaveWithDup : boolean;
var
  i : byte;
  ext : TFileName;
  j : SmallInt;
begin
  i := 0;
  while i < MAX_EXE_NUM do begin
    ext := GetItem (SourceEdit.Text, i, ';');
    if ext = '' then break;//makes result = true

    j := MainForm.ExeOpts.IndexOfSource (ext) + 1;
    if (j > 0) and (j <> CurRec) then begin
      result := MessageDlg ('Extension "' + ext + '" is already in record #' + IntToStr (j) + '. Do you still wish to save this record?', mtWarning, [mbYes, mbNo], 0) = mrYes;
      exit;
    end;

    inc (i);
  end;
  result := true;
end;

procedure TExeOptForm.ShowCurRec;
begin
  if CurRec > NumOfRecs then CurRec := NumOfRecs;
  if CurRec = 0 then CurRec := 1;

  RecNumSB.Position := CurRec;
  RecNumEdit.Text := IntToStr (CurRec);

  with MainForm.ExeOpts.Items[CurRec - 1]^ do
    if CurRec <= MainForm.ExeOpts.GetCount then begin
      SourceEdit.Text := SourceExts.CommaText;
      ExeEdit.Text := ExeExt;
      ParamsPromptCB.Checked := ParamsPrompt;
      cdCB.Checked := cd;
      SaveOrBackupRG.ItemIndex := SaveOrBackupIndex;

      modified := false;
      EnableButtons;
    end else ClearBtn.Click;
end;

procedure TExeOptForm.SetNumOfRecs (num : Cardinal);
begin
  NumOfRecs := num;
  if num = 0 then
    RecNumSB.Max := RecNumSB.Min
  else
    RecNumSB.Max := num;
end;

procedure TExeOptForm.FormShow(Sender: TObject);
var i : SmallInt;
begin
  NumOfRecs := MainForm.ExeOpts.GetCount;
  modified := false;
  HotKey.HotKey := MainForm.ExecuteCurrentFileMnu.ShortCut;

  i := MainForm.ExeOpts.IndexOfSource (ExtractFileExt (MainForm.FileName)) + 1;
  if i = 0 then begin
    CurRec := NumOfRecs;//so IsSaved will return true
    NewBtn.Click
  end else
    ChangeCurRec (i);

  SourceEdit.SetFocus;
end;

procedure TExeOptForm.ClearBtnClick(Sender: TObject);
begin
  SourceEdit.Text := '';
  ExeEdit.Text := '';
  ParamsPromptCB.Checked := false;
  cdCB.Checked := false;
  SaveOrBackupRG.ItemIndex := SaveOrBackupRG.Items.Count - 1;

  modified := IsSaved;
  EnableButtons;
end;

procedure TExeOptForm.SaveBtnClick(Sender: TObject);
begin
  if not SaveWithDup then exit;

  MainForm.ExeOpts.Update (GetRecAsExeOpt, CurRec - 1);
  modified := false;
  EnableButtons;
end;

procedure TExeOptForm.NewBtnClick(Sender: TObject);
begin
  if IsSaved then
    SetNumOfRecs (NumOfRecs + 1)
  else if CurRec = NumOfRecs then exit;

  ChangeCurRec (NumOfRecs);
  modified := false;
end;

procedure TExeOptForm.ReloadBtnClick(Sender: TObject);
begin
  ShowCurRec;
  HotKey.HotKey := MainForm.ExecuteCurrentFileMnu.Shortcut;
end;

procedure TExeOptForm.DeleteBtnClick(Sender: TObject);
begin
  if MainForm.ConfirmPrompt then
    if MessageDlg ('Sure you with to DELETE this record?',
                   mtConfirmation, [mbYes, mbNo], 0)
      = mrNo then exit;

  MainForm.ExeOpts.Delete (CurRec - 1);
  SetNumOfRecs (NumOfRecs - 1);
  ShowCurRec;
end;

procedure TExeOptForm.CloseBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TExeOptForm.NextBtnClick(Sender: TObject);
begin
  ChangeCurRec (CurRec + 1);
end;

procedure TExeOptForm.PreBtnClick(Sender: TObject);
begin
  ChangeCurRec (CurRec - 1);
end;

procedure TExeOptForm.RecNumSBScroll(Sender: TObject;
  ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
  ChangeCurRec (RecNumSB.Position);
end;

procedure TExeOptForm.RecNumEditKeyPress(Sender: TObject; var Key: Char);
begin
  ValidateKey (Key);
  if Key = #13 then begin
    ChangeCurRec (StrToInt (RecNumEdit.Text));
    Key := #0; //prevent beep
  end;
end;

procedure TExeOptForm.SourceEditChange(Sender: TObject);
begin
  SetModified;
end;

procedure TExeOptForm.ExeEditChange(Sender: TObject);
begin
  SetModified;
end;

procedure TExeOptForm.ParamsPromptCBClick(Sender: TObject);
begin
  SetModified;
end;

procedure TExeOptForm.cdCBClick(Sender: TObject);
begin
  SetModified;
end;

procedure TExeOptForm.SaveOrBackupRGClick(Sender: TObject);
begin
  SetModified;
end;

procedure TExeOptForm.ExeEditKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = ';' then begin
    ShowMsgDlg ('Only one extensions (which should be either exe, com, or bat) may be specified here', mtWarning);
    Key := #0;
  end;
end;

procedure TExeOptForm.FormHide(Sender: TObject);
begin
  PromptToSave;
  MainForm.ExecuteCurrentFileMnu.ShortCut := HotKey.HotKey;
end;

procedure TExeOptForm.RecNumEditExit(Sender: TObject);
var i : integer;
begin
  i := StrToInt (RecNumEdit.Text);
  if (i < 1) or (i > NumOfRecs) then
    RecNumEdit.Text := IntToStr (CurRec)
  else
    ChangeCurRec (i);
end;

procedure TExeOptForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_F1 then HelpBtn.Click;
end;

end.

