unit FiltersUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, StdCtrls, Grids, Menus, ExtCtrls;

const DIVIDER = ' | ';

type
  TFiltersForm = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    OpenSavePopup: TPopupMenu;
    InsertMI: TMenuItem;
    DeleteMI: TMenuItem;
    FiltersSG: TStringGrid;
    InsertBtn: TBitBtn;
    DeleteBtn: TBitBtn;
    OrderBtn: TBitBtn;
    Bevel1: TBevel;
    GroupBox2: TGroupBox;
    DefaultExtEdit: TEdit;
    procedure OKBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FiltersSGEnter(Sender: TObject);
    procedure FiltersSGExit(Sender: TObject);
    procedure InsertBtnClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure OrderBtnClick(Sender: TObject);
    procedure InsertMIClick(Sender: TObject);
    procedure FiltersSGMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure DeleteMIClick(Sender: TObject);
  private
    { Private declarations }
    procedure FilterToGrid (const Filter : string);
    function GridToFilter : string;
    function RowsToString (aRow : integer) : string;
    function StringToRows (aRow : string) : TStringList;
  public
    { Public declarations }
  end;

var
  FiltersForm: TFiltersForm;

implementation

uses MainUnit, OrderUnit, ReuUnit, CE_Unit;

{$R *.DFM}

function TFiltersForm.RowsToString (aRow : integer) : string;
var i : integer;
begin
  with FiltersSG do begin
    if (aRow < 0) or (aRow >= RowCount) then exit;

    result := Cells[0, aRow];
    for i := 1 to ColCount - 1 do
      if Cells[i, aRow] = '' then
        break
      else
        result := result + DIVIDER + Cells[i, aRow];

  end;//with...
end;

function TFiltersForm.StringToRows (aRow : string) : TStringList;
var i, DivLen : cardinal;
begin
  result := TStringList.Create;
  i := pos (DIVIDER, aRow);
  if i = 0 then begin
    result.Add (aRow);
    exit;
  end;

  result.Add (copy (aRow, 1, i - 1));
  DivLen := length (DIVIDER);
  result.Add (copy (aRow, i + DivLen, length (aRow) - i - DivLen + 1));
end;

procedure TFiltersForm.FilterToGrid (const Filter : string);
var start, stop, len, aRow, aCol : cardinal;
begin
  with FiltersSG do begin
    Cells[0, 0] := 'Filter Name';
    Cells[1, 0] := 'Filter';
    len := length (Filter);
    if len = 0 then exit;
    start := 1;
    aRow := 1;
    aCol := 0;
    stop := 1;
    while stop < len do begin
      stop := posL ('|', Filter, start, len);
      if stop = 0 then stop := len + 1;

      Cells [aCol, aRow] := copy (Filter, start, stop - start);

      start := stop + 1;
      inc (aCol);
      if aCol = ColCount then begin
        aCol := 0;
        inc (aRow);
        if aRow > RowCount then exit;
      end;
    end; // while...

    {Set the rest of FiltersSG's strings to
     ''}
    while aRow < RowCount do begin
      Cells[aCol, aRow] := '';
      inc (aCol);
      if aCol = ColCount then begin
        aCol := 0;
        inc (aRow);
      end; // if...
    end; // while...

  end; // with StringGrid...
end;

function TFiltersForm.GridToFilter : string;
var aRow, aCol : cardinal;
begin
  {Copy each string of FiltersSG to Filter
   starting at the second row (so column
   titles aren't copied) and putting a '|'
   after each string}
  with FiltersSG do
    for aRow := 1 to RowCount - 1 do
      //don't insert a '|' in a blank row
      if Rows[aRow].Text <> '' then
        for aCol := 0 to ColCount - 1 do
          result := result + Cells[aCol, aRow] + '|';

  //delete probible trailing '|'
  aCol := length (result);
  while (result[aCol] = '|') and (aCol >= 1) do begin
    delete (result, aCol, 1);
    dec (aCol);
  end;
end;

procedure TFiltersForm.OKBtnClick(Sender: TObject);
begin
  with MainForm do begin
    Filter := GridToFilter;

    {Erase beginning . if there is one}
    if DefaultExtEdit.GetTextLen > 0 then
      if pos('.', DefaultExtEdit.Text[1]) = 1 then
        DefaultExtEdit.Text := copy (DefaultExtEdit.text, 2, DefaultExtEdit.GetTextLen - 1);

    DefaultExt := DefaultExtEdit.Text;
  end;
end;

procedure TFiltersForm.FormShow(Sender: TObject);
begin
  with MainForm do begin;
    FilterToGrid (Filter);
    DefaultExtEdit.Text := DefaultExt;
  end;
end;

procedure TFiltersForm.FiltersSGEnter(Sender: TObject);
begin
  FiltersSG.Options := FiltersSG.Options + [goAlwaysShowEditor];
end;

procedure TFiltersForm.FiltersSGExit(Sender: TObject);
begin
  FiltersSG.Options := FiltersSG.Options - [goAlwaysShowEditor];
end;

procedure TFiltersForm.InsertBtnClick(Sender: TObject);
var aRow, aCol : cardinal;
begin
  with FiltersSG do begin
    //confirm is insertion will delete last row
    if Cells[0, RowCount - 1] <> '' then
      if MessageDlg ('Inserting a new row will delete the last row. Do you want to insert a new row anyway?', mtConfirmation, [mbYes, mbNo], 0) = mrNo then
        exit;

    //move rows up to selected row down 1 row
    {- 2 because I want to start at second to last row and
     because Cells is 0 based}
    for aRow := RowCount - 2 downto Row do
      for aCol := 0 to ColCount - 1 do
        Cells[aCol, aRow + 1] := Cells[aCol, aRow];

    //clear out selected row
    for aCol := 0 to ColCount - 1 do
      Cells[aCol, Row] := '';
  end;//with...
end;

procedure TFiltersForm.DeleteBtnClick(Sender: TObject);
var aRow, aCol : cardinal;
begin
  with FiltersSG do begin
    //move rows up to selected row up 1 row
    {- 2 because I want to end at second to last row and
     because Cells is 0 based}
    for aRow := Row to RowCount - 2 do
      for aCol := 0 to ColCount - 1 do
        Cells[aCol, aRow] := Cells[aCol, aRow + 1];

    //clear out last row
    for aCol := 0 to ColCount - 1 do
      Cells[aCol, RowCount - 1] := '';
  end;//with...
end;

procedure TFiltersForm.OrderBtnClick(Sender: TObject);
var
  i : cardinal;
  s : string;
begin
  OrderForm.MainItem := -1;

  OrderForm.ListBox1.Clear;
  i := 1;
  while (i < FiltersSG.RowCount) do begin
     s := RowsToString (i);
     if s = '' then break;
     OrderForm.ListBox1.Items.Add (s);
     inc (i);
  end;

  if OrderForm.ShowModal = mrCancel then exit;

  for i := 1 to OrderForm.ListBox1.Items.Count - 1 do
    FiltersSG.Rows[i] := StringToRows (OrderForm.ListBox1.Items[i - 1]);
end;


procedure TFiltersForm.InsertMIClick(Sender: TObject);
begin
  InsertBtn.Click;
end;

//inserts a blank row in FiltersSG
procedure TFiltersForm.FiltersSGMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var aCol, aRow : LongInt;
begin
  {when the user right click a popup menu appears; this
   ensures the cell that was right clicked on is the
   selected cell}
  if Button = mbRight then begin
    FiltersSG.MouseToCell (X, Y, aCol, aRow);
    if aRow > 0 then begin //don't select Col headers
      FiltersSG.Col := aCol;
      FiltersSG.Row := aRow;
    end;
  end;
end;

procedure TFiltersForm.DeleteMIClick(Sender: TObject);
begin
  DeleteBtn.Click;
end;







end.
