unit NewUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, Buttons,
  ReuUnit, Grids, Outline, DirOutln;

type
  TNewForm = class(TForm)
    FileTypesLB: TListBox;
    DefaultTextMemo: TMemo;
    Label1: TLabel;
    Label2: TLabel;
    Bevel1: TBevel;
    CancelBtn: TBitBtn;
    OKBtn: TBitBtn;
    Label3: TLabel;
    FileNameEdit: TEdit;
    UseDefaultCB: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FileTypesLBClick(Sender: TObject);
    procedure FileNameEditChange(Sender: TObject);
    procedure FileTypesLBDblClick(Sender: TObject);
    procedure SetDefaultTextBtnClick(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure FileNameEditEnter(Sender: TObject);
    procedure FileNameEditClick(Sender: TObject);
    procedure DefaultTextMemoKeyPress(Sender: TObject; var Key: Char);
    procedure FileNameEditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FileTypesLBKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    Extensions : TStringList;
    JustEnteredEdit : boolean;//see FileNameEditClick
    
    procedure LoadDefaultText;
    procedure SaveDefaultText;

  public
    { Public declarations }

    function FileName : TFileName;
  end;

var
  NewForm: TNewForm;

implementation

uses MainUnit;

{$R *.DFM}

function TNewForm.FileName : TFileName;
begin
  result := FileNameEdit.Text;
end;

procedure TNewForm.LoadDefaultText;
begin
  with DefaultTextMemo do begin
    Text := MainForm.TextDefaults.GetText(ExtractFileExt(FileNameEdit.Text));
    Modified := false;
  end;
end;

procedure TNewForm.SaveDefaultText;
begin
  with DefaultTextMemo do
    if Modified then
      MainForm.TextDefaults.SetText(ExtractFileExt(FileNameEdit.Text), Text);
end;

procedure TNewForm.FormShow(Sender: TObject);
var i : integer;
begin
  UseDefaultCB.Checked := true;
  
  DefaultTextMemo.Clear;//so current text won't be saved
  DefaultTextMemo.Font.Name := MainForm.Edit.Font.Name;
  //so something in FileTypesLB will be selected
  FileNameEdit.Clear;

  //Extensions used as temp
  FilterTypesToStringList(Extensions, MainForm.Filter);
  FileTypesLB.Items := Extensions;

  FilterExtensionsToStringList(Extensions, MainForm.Filter, false);

  with Extensions do
    if Count > 0 then
      for i := 0 to Count - 1 do
        if Strings[i] = '.*' then begin
          Strings[i] := '';
          FileTypesLB.Items[i] := 'None';
        end;

  FileNameEdit.Text := MainForm.NextUntitledName;
  FileNameEdit.SetFocus;
  //ensure FileNameEdit has a selection
  FileNameEditEnter(Sender);
end;

procedure TNewForm.FormCreate(Sender: TObject);
begin
  Extensions := TStringList.Create;
end;

procedure TNewForm.FormDestroy(Sender: TObject);
begin
  Extensions.Free;
end;

procedure TNewForm.FileTypesLBClick(Sender: TObject);
begin
  with FileNameEdit do begin
    SaveDefaultText;
    Text := ChangeExt(Text, Extensions[FileTypesLB.ItemIndex]);
    LoadDefaultText;
  end;
end;

procedure TNewForm.FileNameEditChange(Sender: TObject);
var
  i : integer;
  ext : TFileName;
begin
  ext := ExtractFileExt(FileNameEdit.Text);
  i := Extensions.IndexOf(ext);
  if i = -1 then i := 0;
  if i <> FileTypesLB.ItemIndex then begin
    SaveDefaultText;
    FileTypesLB.ItemIndex := i;
    LoadDefaultText;
  end;
end;

procedure TNewForm.FileTypesLBDblClick(Sender: TObject);
begin
  OKBtn.Click;
end;

procedure TNewForm.SetDefaultTextBtnClick(Sender: TObject);
begin
  MainForm.TextDefaults.SetText(ExtractFileExt(FileNameEdit.Text), DefaultTextMemo.Text);
end;

procedure TNewForm.FormHide(Sender: TObject);
begin
  SaveDefaultText;
end;

procedure TNewForm.FileNameEditEnter(Sender: TObject);
begin
  //highlight file name up to the extension
  with FileNameEdit do begin
    SelStart := 0;
    SelLength := GetTextLen - length(ExtractFileExt(Text));
  end;
  JustEnteredEdit := true;
end;

procedure TNewForm.FileNameEditClick(Sender: TObject);
begin
  {When the user enters FileNameEdit with a mouse click,
   the mouse click delselects the text after
   FileNameEditEnter is called. This code compensates for
   that.}
  if JustEnteredEdit then begin
    FileNameEditEnter(Sender);
    JustEnteredEdit := false;
  end;
end;

procedure TNewForm.DefaultTextMemoKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #27 then CancelBtn.Click;
end;

procedure TNewForm.FileNameEditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var hSS, hSL : integer;
begin
  hSS := FileNameEdit.SelStart;
  hSL := FileNameEdit.SelLength;

  with FileTypesLB do
    if Key = VK_DOWN then
      ItemIndex := (ItemIndex + 1) mod Items.Count
    else if Key = VK_UP then
      ItemIndex := (ItemIndex - 1 + Items.Count) mod Items.Count
    else exit;

  FileTypesLB.OnClick(Sender);
  FileNameEdit.SelStart := hSS;
  FileNameEdit.SelLength := hSL;
  Key := 0;//if Key = VK_DOWN or VK_UP
end;

procedure TNewForm.FileTypesLBKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  with FileTypesLB do
    if (Key = VK_DOWN) and (ItemIndex = Items.Count - 1) then begin
      ItemIndex := 0;
      Key := 0;
    end else if (Key = VK_UP) and (ItemIndex = 0) then begin
      ItemIndex := Items.Count - 1;
      Key := 0;
    end;
end;

end.
