unit OrderUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ReuUnit;

type
  TOrderForm = class(TForm)
    ListBox1: TListBox;
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    SortBtn: TBitBtn;
    UpBtn: TButton;
    DownBtn: TButton;
    procedure SortBtnClick(Sender: TObject);
    procedure ListBox1DragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure UpBtnClick(Sender: TObject);
    procedure DownBtnClick(Sender: TObject);
    procedure ListBox1DragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure ListBox1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ListBox1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormShow(Sender: TObject);
    procedure ListBox1KeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ListBox1Enter(Sender: TObject);
    procedure ListBox1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    { Private declarations }
    function FindSelStart : integer;
    function FindSelStop : integer;
    procedure SetEnables;
    function SwapItems (Item : integer) : boolean;
  public
    { Public declarations }
    MainItem : integer;
  end;

var
  OrderForm: TOrderForm;

implementation

{$R *.DFM}

function TOrderForm.FindSelStart : integer;
begin
  with ListBox1 do
    if Items.Count > 0 then
      for result := 0 to Items.Count - 1 do
        if Selected[result] then exit;

  result := -1;
end;

function TOrderForm.FindSelStop : integer;
begin
  with ListBox1 do
    if Items.Count > 0 then
      for result := Items.Count - 1 downto 0 do
        if Selected[result] then exit;

  result := -1;
end;

procedure TOrderForm.SetEnables;
var fss : integer;
begin
  with ListBox1 do begin
    fss := FindSelStart;
    UpBtn.Enabled := fss > 0;
    DownBtn.Enabled := (fss <> -1) and (fss + SelCount < Items.Count);
  end;
end;

function TOrderForm.SwapItems (Item : integer) : boolean;

  function Valid (i : integer) : boolean;
  begin
    result := (i >= 0) and (i < ListBox1.Items.Count);
  end;

  var i, SelStart, SelStop : integer;
begin
  with ListBox1 do begin
    SelStart := FindSelStart;
    SelStop := FindSelStop;

    result := (ItemIndex <> Item) and Valid (ItemIndex)
      and Valid (Item) and (SelStart <> -1) and
      ((Item < SelStart) or (Item > SelStop));
    if not result then exit;

    if SelStart < Item then begin
      for i := SelStart to SelStop do
        Items.Move(SelStart, Item);

      if (SelStart <= MainItem) and (MainItem <= SelStop) then
        MainItem := MainItem + Item - SelStop
      else if (SelStop < MainItem) and (MainItem <= Item) then
        MainItem := MainItem - (SelStop - SelStart + 1);

      i := SelStop - SelStart;
      SelStop := SelStop - SelStart + Item - i;
      SelStart := Item - i;
    end else begin
      for i := SelStart to SelStop do
        Items.Move(i, Item + i - SelStart);

      if (Item <= MainItem) and (MainItem < SelStart) then
        MainItem := MainItem + (SelStop - SelStart + 1)
      else if (SelStart <= MainItem) and (MainItem <= SelStop) then
        MainItem := MainItem + Item - SelStart;

      SelStop := Item + SelStop - SelStart;
      SelStart := Item;
    end;

    for i := 0 to Items.Count - 1 do
      Selected[i] := (SelStart <= i) and (i <= SelStop);

    SetEnables;
  end;//with...
end;

procedure TOrderForm.SortBtnClick(Sender: TObject);

  function MainItemExists : boolean;
  begin
    result := (MainItem >= 0) and (MainItem < ListBox1.Items.Count);
  end;

var hMainItem : string;
begin
  if MainItemExists then
    hMainItem := ListBox1.Items[MainItem];

  ListBox1.Sorted := true;
  ListBox1.Sorted := false;

  if MainItemExists then begin
    //put MainItem at top of sort
    ListBox1.Items.Delete (ListBox1.Items.IndexOf (hMainItem));
    ListBox1.Items.Insert (0, hMainItem);
    MainItem := 0;
  end;
end;

procedure TOrderForm.ListBox1DragDrop(Sender, Source: TObject; X,
  Y: Integer);
begin
  SwapItems (ListBox1.ItemAtPos (Point (X, Y), true));
end;

procedure TOrderForm.UpBtnClick(Sender: TObject);
begin
  with ListBox1 do
    if ItemIndex = 0 then begin
      //SwapItems (Items.Count - 1)
      Items.Add (Items[0]);
      Items.Delete (0);
      ItemIndex := Items.Count - 1;
      SetEnables;
    end else if not SwapItems (FindSelStart - 1) then
      ItemIndex := 0;
end;

procedure TOrderForm.DownBtnClick(Sender: TObject);
begin
  with ListBox1 do
    if ItemIndex = Items.Count - 1 then begin
      Items.Insert (0, Items[Items.Count - 1]);
      Items.Delete (Items.Count - 1);
      ItemIndex := 0;
      //SwapItems (0)
      SetEnables;
    end else if not SwapItems (FindSelStart + SelCount) then
      ItemIndex := Items.Count - 1;
end;

procedure TOrderForm.ListBox1DragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  {needs remark because for some reason that makes
   drag work right}
end;

procedure TOrderForm.ListBox1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button <> mbLeft then exit;
  if ListBox1.SelCount > 0 then ListBox1.DragMode := dmAutomatic;
  SetEnables;
end;

procedure TOrderForm.ListBox1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if ssCtrl in Shift then begin
    Shift := Shift - [ssCtrl] + [ssShift];
    ListBox1MouseDown(Sender, Button, Shift, X, Y);
  end;
  ListBox1.DragMode := dmManual;
end;

procedure TOrderForm.FormShow(Sender: TObject);
begin
  SetEnables;
end;

procedure TOrderForm.ListBox1KeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  SetEnables;
end;

procedure TOrderForm.ListBox1Enter(Sender: TObject);
begin
  ListBox1.Tag := -1;
end;

procedure TOrderForm.ListBox1MouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  ListBoxPosToHint(ListBox1, Application, X, Y);
end;

end.
