unit ReuUnit;

interface

uses SysUtils, Dialogs, StdCtrls, Classes, Forms, ShellAPI;

const
  CR = #13#10;
  PB = CR + CR;

function ChangeExt (const fName, ext : TFileName) : TFileName;
function CharToDriveNum (ch : char) : byte;
function CommonPart(const s1, s2 : string; caseSens : boolean) : string;
procedure CropLabel (var lab : TLabel);
function FileExistsEx(const fName : TFileName; requireFullyQualified : boolean) : boolean;
procedure FilterExtensionsToStringList (var sl : TStringList; const filter : string; full : boolean);
procedure FilterTypesToStringList (var sl : TStringList; const filter : string);
function FullyQualified(const fName : TFileName) : boolean;
function GetDriveLetter (const fName : TFileName) : char;
function GetFileTime(const fName : TFileName; var time : integer) : boolean;
function GetFilterItemIndex (const item, filter : string) : integer;
function GetCRCount(const line : string) : integer;
function HiWord(li : LongInt) : word;
function IndexOf(const sl : TStrings; const s : string; matchCase : boolean) : integer;
function IntToBool (i : integer): boolean;
function IsAlpha(ch : char) : boolean;
function IsAlphaNum(ch : char) : boolean;
function IsDigit(ch : char) : boolean;
function IsWhiteSpace (ch : char) : boolean;
procedure LoadStringListFromFile(var sl : TStringList; var FileVar : TextFile);
function LongNameOf (const fName : TFileName) : TFileName;
function LoWord(li : LongInt) : word;
procedure ListBoxPosToHint(var ListBox : TListBox; const Application : TApplication; X, Y : integer);
function LTrim (const str : string) : string;
function max (i1, i2 : integer) : integer;
function min (i1, i2 : integer) : integer;
function posE (ch : char; const str : string) : integer;
function posI (const subStr, str : string) : integer;
function posL (const subStr, str : string; start, stop : cardinal) : integer;
function posN (const subStr, str : string; n : integer) : integer;
function RecycleFile(const sFileName : string) : boolean;
function Replace (const src1, src2 : string; var dest : string) : boolean;
function ReplaceAll (const src1, src2 : string; var dest : string) : integer;
procedure SaveStringListToFile(const sl : TStringList; var FileVar : TextFile);
function ShortNameOf (const fName : TFileName) : TFileName;
procedure ShowMsgDlg (const msg : string; aType : TMsgDlgType);
function StrCmp (const s1, s2 : string) : boolean;
function StrCmpI (const s1, s2 : string) : boolean;
function TabsToSpaces(const str : string; spaceCount : integer) : string;
function TitleCase (const str : string) : string;
function ToggleCase (const str : string) : string;
function tolower (ch : char) : char;
function toupper (ch : char) : char;

implementation

function ChangeExt (const fName, ext : TFileName): TFileName;
var i : cardinal;
begin
  i := posE ('.', fName);
  if i > 0 then
    result := copy (fName, 1, i - 1)
  else
    result := fName;

  if (ext <> '') and (ext[1] <> '.') then
    result := result + '.' + ext
  else
    if ext <> '.' then result := result + ext;
end;

//returns 'a' or 'A' as 1, 'b' or 'B' as 2...
function CharToDriveNum (ch : char) : byte;
begin
  result := Ord (UpCase (ch)) - 64;
end;

function CommonPart(const s1, s2 : string; caseSens : boolean) : string;

  function reCase(ch : char) : char;
  begin
    if caseSens then
      result := ch
    else
      result := tolower(ch);
  end;

var i, minLen : integer;
begin
  minLen := min(length(s1), length(s2));
  result := '';
  if minLen >= 1 then
    for i := 1 to minLen do
      if reCase(s1[i]) = reCase(s2[i]) then
        result := result + s1[i];
end;

{if label will run off the edge of its parent, crop label
 will make the label fit on its parent and set the hint to
 it's caption; ShowHint will be true iff label was cropped}
procedure CropLabel (var lab : TLabel);
begin
  with lab do begin
    ShowHint := Left + Width > Parent.Width - 4;
    if not ShowHint then exit;
    Hint := Caption;
  end;
end;

//if fully qualified is true, the file must specify a
//complete path
function FileExistsEx(const fName : TFileName; requireFullyQualified : boolean) : boolean;
begin
  //fName must be either not specified as fullyQualified,
  //or actually be FullyQualified, and fName must exist
  result := (not requireFullyQualified or FullyQualified(fName))
            and FileExists(fName);
end;

{fills sl with the part of filter that has the extensions;
 if not full then the * is removed;
 sl is assumed to have already been Created}
procedure FilterExtensionsToStringList (var sl : TStringList; const filter : string; full : boolean);
var i1, i2, i3, len : integer;
begin
  if full then i3 := 0 else i3 := 1;

  sl.Clear;
  len := length (filter);
  if len = 0 then exit;
  i2 := 0;

  repeat
    i1 := i2 + 1;
    i2 := posL ('|', filter, i1, len);
    if i2 = 0 then exit;
    i1 := i2 + 1;
    i2 := posL ('|', filter, i1, len);
    if i2 = 0 then i2 := len + 1;
    sl.Add (copy (filter, i1 + i3, i2 - i1 - i3));
  until i2 = 0;
end;

{fills sl with the part of filter that has the file types;
 sl is assumed to have already been Created}
procedure FilterTypesToStringList (var sl : TStringList; const filter : string);
var i1, i2, len : integer;
begin
  sl.Clear;
  len := length (filter);
  i2 := 0;

  repeat
    i1 := i2 + 1;
    i2 := posL ('|', filter, i1, len);
    if i2 > 0 then sl.Add (copy (filter, i1, i2 - i1));
    i1 := i2 + 1;
    i2 := posL ('|', filter, i1, len);
  until i2 = 0;
end;

//returns true iff a fName has a path of some kind
function FullyQualified(const fName : TFileName) : boolean;
var len : integer;
begin
  len := length(fName);
  result := (len >= 2) and ((pos(':/', fName) > 1) or (pos(':\', fName) > 1) or (Copy(fName, 1, 2) = '//'));
end;

function GetDriveLetter (const fName : TFileName) : char;
begin
  assert(length(fName) >= 2);
  assert(fName[2] = ':');
  result := fName[1];
end;

function GetFileTime(const fName : TFileName; var time : integer) : boolean;
var sr : TSearchRec;
begin
  result := FindFirst(fName, faAnyFile, sr) = 0;
  time := sr.Time;
end;

{GetFilterItemIndex ('*.jpeg',
 'All Files|*.*|JPEG (jpg, jpeg)|*.jpg;*.jpeg')
 returns 2;
 note JPEG (jpg, jpeg) cannot be JPEG (*.jpg, *.jpeg);
 note index is 1 based, but TStringList is 0 based}
function GetFilterItemIndex (const item, filter : string) : integer;
var sl : TStringList;
begin
  sl := TStringList.Create;
  FilterExtensionsToStringList (sl, filter, true);
  result := sl.IndexOf (item) + 1;
  sl.Free;
end;

function GetCRCount(const line : string) : integer;
var i, len : integer;
begin
  result := 0;
  len := length(line);
  if len = 0 then exit;

  for i := 1 to len do
    if line[i] = #13 then inc(result);
end;

function HiWord(li : LongInt) : word;
begin
  result := li shr (SizeOf(LongInt) - SizeOf(word));
end;

function IndexOf(const sl : TStrings; const s : string; matchCase : boolean) : integer;
  {
  function BinarySearch : integer;
  var
    i, min, max : integer;
    StrCmp : function(pc1, pc2 : PChar) : integer;
  begin
    if matchCase then
      StrCmp := StrComp
    else
      StrCmp := StrIComp;

    with sl do begin
      min := 0; max := Count - 1;
      while (max - min > 1) do begin
        result := (max + min) div 2;
        i := StrCmp(PChar(Strings[result]), PChar(s));
        if i = 0 then exit;
        if i < 0 then min := result;
        if i > 0 then max := result;
      end;
      if StrCmp(PChar(Strings[max]), PChar(s)) = 0 then
        result := max
      else if StrCmp(PChar(Strings[min]), PChar(s)) = 0 then
        result := min
      else
        result := -1;
    end;//with
  end;
  }
  function LinearSearch : integer;
  begin
    with sl do
      if Count > 0 then
        for result := 0 to Count - 1 do
          if (matchCase and StrCmp(Strings[result], s)) or
             (not matchCase and StrCmpI(Strings[result], s)) then begin
             exit;
          end;

    result := -1;
  end;

begin
{  if sorted then
    result := BinarySearch
  else}
  result := LinearSearch;
end;

function IntToBool (i : integer): boolean;
begin
  result := i <> 0;
end;

function IsAlpha(ch : char) : boolean;
begin
  result := ch in ['A'..'Z','a'..'z'];
end;

function IsAlphaNum(ch : char) : boolean;
begin
  result := IsAlpha(ch) or IsDigit(ch);
end;

function IsDigit(ch : char) : boolean;
begin
  result := ch in ['0'..'9'];
end;

function IsWhiteSpace (ch : char) : boolean;
begin
  result := ch in [' ',#9,#13,#10];
end;

procedure LoadStringListFromFile(var sl : TStringList; var FileVar : TextFile);
var
  i : integer;
  inStr : string;
begin
  ReadLn(FileVar, i);
  sl.Clear;
  while (sl.Count < i) do begin
    ReadLn(FileVar, inStr);
    sl.Add(inStr);
  end;
end;

{returns long file name version of fName;
 an exception will be raised if fName
 does not exist; fName can be a path}
function LongNameOf (const fName : TFileName) : TFileName;
var
  SearchRec : TSearchRec;
  i : integer;
  path : string;
begin
  i := length (fName);
  if i = 0 then exit;
  //get rid of possible trailing \
  if fName[i] = '\' then dec (i);
  result := copy (fName, 1, i);

  //FindFirst returns 0 iff successful
  i := FindFirst (result, faAnyFile, SearchRec);
  while (i = 0) and ((SearchRec.Name = '.') or (SearchRec.Name = '..')) do
    i := FindNext(SearchRec);

  if i = 0 then begin
    path := ExtractFilePath(result);
    //I would get cought in an infinite recursion if
    //path = result
    if path <> result then
      result := LongNameOf(path) + '\' + SearchRec.Name;

    FindClose(SearchRec);
  end else if length (result) > 2 then begin
    FindClose(SearchRec);
    raise Exception.Create ('cannot convert "' + fName + '" to long file name');
  end;
end;

function LoWord(li : LongInt) : word;
begin
  result := li;
end;

{uses Tag as Last ItemAtPos(X,Y); should be set to -1
 OnEnter}
procedure ListBoxPosToHint(var ListBox : TListBox; const Application : TApplication; X, Y : integer);
var i : integer;
begin
  with ListBox do begin
    i := ItemAtPos(Point(X, Y), true);
    if i <> Tag then begin
      //the hint won't change with out calling CancelHint
      Application.CancelHint;
      if i = -1 then
        Hint := ''
      else
        Hint := Items[i];
    end;
    Tag := i;
  end;
end;

function LTrim (const str : string) : string;
var i, len : integer;
begin
  len := length (str);
  i := 1;
  while (i <= len) and ((str[i] = ' ') or (str[i] = #9)) do inc (i);
  result := copy (str, i, length (str) - i + 1);
end;

function max (i1, i2 : integer) : integer;
begin
  if i1 > i2 then
    result := i1
  else
    result := i2;
end;

function min (i1, i2 : integer) : integer;
begin
  if i1 < i2 then
    result := i1
  else
    result := i2;
end;

{end pos; returns index of rightmost
 occurance of ch in str}
function posE (ch : char; const str : string) : integer;
var i, len : cardinal;
begin
  len := length (str);

  if len = 0 then begin
    result := 0;
    exit;
  end;

  for i := len downto 1 do
    if ch = str[i] then begin
      result := i;
      exit;
    end;

  result := 0;
end;

//case insensitive pos
function posI (const subStr, str : string) : integer;
begin
  result := pos (LowerCase (subStr), LowerCase (str));
end;

{returns 1 based position of subStr in Str if subStr is
 contained b/t start and stop; otherwise 0 is returned}
function posL (const subStr, str : string; start, stop : cardinal) : integer;
var len : cardinal;
begin
  len := length (subStr);
  if stop - start + 1 < len then begin
    result := 0;
    exit;
  end;

  for start := start to stop - len + 1 do
    if subStr[1] = Str[start] then
      if subStr = copy (Str, start, len) then begin
        result := start;
        exit;
      end;

  result := 0;
end;

//returns pos of nth occurance of subStr in str
function posN (const subStr, str : string; n : integer) : integer;
  var len : integer;

  function DoPosN (start, n : integer) : integer;
  begin
    result := posL (subStr, str, start, len);
    if (n = 1) or (result = 0) then exit;
    result := DoPosN (result + 1, n - 1);
  end;

begin
  assert (n > 0, 'posN called with n <= 0');
  len := length (str);
  result := DoPosN (1, n);
end;

//from http://www.chami.com/tips/delphi/120196D.html
function RecycleFile(const sFileName : string) : boolean;
var fos : TSHFileOpStruct;
begin
  //FillChar(fos, SizeOf(fos), 0);
  with fos do begin
    Wnd := 0;
    wFunc := FO_DELETE;
    pFrom := PChar(sFileName);
    fFlags := FOF_ALLOWUNDO
              or FOF_NOCONFIRMATION
              or FOF_SILENT;
  end;
  result := 0 = ShFileOperation(fos);
end;

function Replace (const src1, src2 : string; var dest : string) : boolean;
var i : integer;
begin
  i := pos (src1, dest);
  result := i > 0;
  if not result then exit;
  delete (dest, i, length (src1));
  insert (src2, dest, i);
end;

function ReplaceAll (const src1, src2 : string; var dest : string) : integer;
begin
  result := 0;
  while (Replace (src1, src2, dest)) do inc (result);
end;

procedure SaveStringListToFile(const sl : TStringList; var FileVar : TextFile);
var i : integer;
begin
  WriteLn(FileVar, sl.Count);
  if sl.Count > 0 then
    for i := 0 to sl.Count - 1 do
      WriteLn(FileVar, sl[i]);
end;

{returns short file name version of fName;
 an exception will be raised if fName
 does not exist; fName can be a path}
function ShortNameOf (const fName : TFileName) : TFileName;
var SearchRec : TSearchRec;

  {if cAlternateFileName = cFileName, then
   cAlternateFileName sometimes = nil}
  function NotNullName : string;
  begin
    if TFileName (SearchRec.FindData.cAlternateFileName) = '' then
      result := TFileName (SearchRec.FindData.cFileName)
    else
      result := TFileName (SearchRec.FindData.cAlternateFileName);
  end;

var i : integer;
begin
  //get rid of possible trailing \
  i := length (fName);
  if i = 0 then exit;
  if fName[i] = '\' then dec (i);
  result := copy (fName, 1, i);
  if i <= 2 then exit;

  //FindFirst returns 0 iff successful
  i := FindFirst (result, faAnyFile, SearchRec);
  {SearchRec.FindData.cAlternateFileName is DOS file name
   of fName as PChar without path}
  if i = 0 then
    result := ShortNameOf (ExtractFilePath (result)) + '\' + NotNullName
  else if length (result) > 2 then
    raise Exception.Create ('cannot convert "' + fName + '" to short file name')
  else result := UpperCase (result);
end;

procedure ShowMsgDlg (const msg : string; aType : TMsgDlgType);
begin
  MessageDlg (msg, aType, [mbOK], 0);
end;

function StrCmp (const s1, s2 : string) : boolean;
begin
  result := StrComp (PChar (s1), PChar (s2)) = 0;
end;

function StrCmpI (const s1, s2 : string) : boolean;
begin
  result := StrIComp (PChar (s1), PChar (s2)) = 0;
end;

function TabsToSpaces(const str : string; spaceCount : integer) : string;
var i, len : integer;
begin
  len := length(str);
  if len = 0 then begin
    result := str;
    exit;
  end;

  result := '';
  for i := 1 to len do
    if str[i] = #9 then
      result := result + StringOfChar(' ', spaceCount)
    else
      result := result + str[i];
end;

function TitleCase (const str : string) : string;
var len, i : integer;
begin
  result := '';
  len := length (str);
  if len = 0 then exit;

  result := result + toupper (str[1]);
  for i := 2 to len do
    if (IsWhiteSpace (str[i - 1])) or (str[i - 1] = '_') then
      result := result + toupper (str[i])
    else
      result := result + tolower (str[i]);
end;

function ToggleCase (const str : string) : string;
var len, i : integer;
begin
  result := '';
  len := length (str);

  for i := 1 to len do
    if ('a' <= str[i]) and (str[i] <= 'z') then
      result := result + toupper(str[i])
    else
      result := result + tolower(str[i]);
end;

function tolower (ch : char) : char;
begin
  if ch in ['A'..'Z'] then
    result := chr (ord (ch) + ord ('a') - ord ('A'))
  else
    result := ch;
end;

function toupper (ch : char) : char;
begin
  if ch in ['a'..'z'] then
    result := chr (ord (ch) - ord ('a') + ord ('A'))
  else
    result := ch;
end;

end.
