unit SyntaxHighlightingUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ColorGrd, ComCtrls, ExtCtrls, CE_Unit,
  ReuUnit, ReservedWordsUnit;

const HE_NONE = 0; HE_KW = 1; HE_S = 2;//for HitEnter

type
  TSavePrompt = (spNo, spYes, spNotAsked);

  TSyntaxHighlightingForm = class(TForm)
    GroupBox1: TGroupBox;
    Label1: TLabel;
    KeyWordsLB: TListBox;
    RemoveKWBtn: TBitBtn;
    CaseSensitiveKWCB: TCheckBox;
    SymbolsGB: TGroupBox;
    Label2: TLabel;
    SymbolsLB: TListBox;
    RemoveSBtn: TBitBtn;
    AddKWMemo: TMemo;
    AddSMemo: TMemo;
    GroupBox2: TGroupBox;
    RecNumSB: TScrollBar;
    NextBtn: TBitBtn;
    PrevBtn: TBitBtn;
    GroupBox5: TGroupBox;
    NewBtn: TBitBtn;
    DeleteBtn: TBitBtn;
    SaveBtn: TBitBtn;
    ReloadBtn: TBitBtn;
    GroupBox3: TGroupBox;
    Bevel1: TBevel;
    ExtensionsEdit: TEdit;
    RecNumMemo: TMemo;
    CaseSensitiveSCB: TCheckBox;
    GroupBox4: TGroupBox;
    GroupBox6: TGroupBox;
    KeyWordsCG: TColorGrid;
    SymbolsCG: TColorGrid;
    CancelBtn: TBitBtn;
    OKBtn: TBitBtn;
    Bevel2: TBevel;
    SyntaxHighlightingOnCB: TCheckBox;
    HelpBtn: TBitBtn;
    procedure KeyWordsCGChange(Sender: TObject);
    procedure SymbolsCGChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure KeyWordsLBKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure SymbolsLBKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure KeyWordsLBKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure KeyWordsLBMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SymbolsLBMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SymbolsLBKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure RemoveKWBtnClick(Sender: TObject);
    procedure RemoveSBtnClick(Sender: TObject);
    procedure AddKWMemoKeyPress(Sender: TObject; var Key: Char);
    procedure RecNumMemoKeyPress(Sender: TObject; var Key: Char);
    procedure NewBtnClick(Sender: TObject);
    procedure RecNumSBScroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
    procedure SaveBtnClick(Sender: TObject);
    procedure ExtensionsEditChange(Sender: TObject);
    procedure CaseSensitiveKWCBClick(Sender: TObject);
    procedure NextBtnClick(Sender: TObject);
    procedure PrevBtnClick(Sender: TObject);
    procedure ReloadBtnClick(Sender: TObject);
    procedure AddSMemoKeyPress(Sender: TObject; var Key: Char);
    procedure CaseSensitiveSCBClick(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure SyntaxHighlightingOnCBClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    FCurRec : integer;
    FModified : boolean;

    procedure Clear;
    function PromptToSave : TSavePrompt;
    function Saveable : boolean;
    procedure SetCurRec(cr : integer);
    procedure SetModified(m : boolean);
    procedure SetEnables;
  public
    { Public declarations }
    CCModified : boolean;//color coding modified

    property CurRec : integer
      read FCurRec
      write SetCurRec;

    property Modified : boolean
      read FModified
      write SetModified;
  end;

var
  SyntaxHighlightingForm: TSyntaxHighlightingForm;

implementation

uses MainUnit;

{$R *.DFM}

procedure TSyntaxHighlightingForm.Clear;
begin
  AddKWMemo.Clear; AddSMemo.Clear;
  CaseSensitiveKWCB.Checked := true;
  CaseSensitiveSCB.Checked := true;
  KeyWordsLB.Clear;
  SymbolsLB.Clear;
  ExtensionsEdit.Clear;
end;

//returns true iff save actually takes place
function TSyntaxHighlightingForm.PromptToSave : TSavePrompt;
begin
  if SaveBtn.Enabled then
    if MessageDlg('Save Current Record?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then begin
      SaveBtn.Click;
      result := spYes;
    end else
      result := spNo
  else
    result := spNotAsked;
end;

{returns true iff there is enough info. filled in to save
 the current record}
function TSyntaxHighlightingForm.Saveable : boolean;
begin
  result := (ExtensionsEdit.Text <> '') and
            ((KeyWordsLB.Items.Count > 0) or
             (SymbolsLB.Items.Count > 0));
end;

procedure TSyntaxHighlightingForm.SetCurRec(cr : integer);
begin
  assert((cr >= 0) and (cr <= RecNumSB.Max));
  if Modified and (PromptToSave = spNo) then
    if MessageDlg('Do you still want to change the record numbre?', mtConfirmation, [mbYes, mbNo], 0) = mrNo then
      exit;

  FCurRec := cr;

  with MainForm do begin
    if (CurRec >= 0) and (CurRec < LangWordsList.Count) then
      with LangWordsList[CurRec]^ do begin
        CaseSensitiveKWCB.Checked := KeyWords.MatchCase;
        KeyWordsLB.Items := KeyWords.Items;

        CaseSensitiveSCB.Checked := Symbols.MatchCase;
        SymbolsLB.Items := Symbols.Items;

        ExtensionsEdit.Text := Extensions.CommaText;
      end
    else Clear;

    RecNumMemo.Text := IntToStr(CurRec + 1);
    RecNumSB.Position := CurRec;
    Modified := false;
    SetEnables;
  end;
end;

procedure TSyntaxHighlightingForm.SetModified(m : boolean);
begin
  FModified := m;
  SetEnables;
end;

procedure TSyntaxHighlightingForm.SetEnables;
begin
  RemoveKWBtn.Enabled := KeyWordsLB.SelCount > 0;
  RemoveSBtn.Enabled := SymbolsLB.SelCount > 0;
  DeleteBtn.Enabled := CurRec < MainForm.LangWordsList.Count;
  ReloadBtn.Enabled := Modified and DeleteBtn.Enabled;
  SaveBtn.Enabled := Modified and Saveable;
  NextBtn.Enabled := CurRec < RecNumSB.Max;
  PrevBtn.Enabled := CurRec > 0;
end;

procedure TSyntaxHighlightingForm.KeyWordsCGChange(Sender: TObject);
begin
  KeyWordsLB.Font.Color := KeyWordsCG.ForegroundColor;
  AddKWMemo.Font.Color := KeyWordsCG.ForegroundColor;
  CCModified := true;
end;

procedure TSyntaxHighlightingForm.SymbolsCGChange(Sender: TObject);
begin
  SymbolsLB.Font.Color := SymbolsCG.ForegroundColor;
  AddSMemo.Font.Color := SymbolsCG.ForegroundColor;
  CCModified := true;
end;

procedure TSyntaxHighlightingForm.FormShow(Sender: TObject);
begin
  CCModified := false;
  Modified := false;
  with MainForm do begin
    with Edit do begin
      KeyWordsLB.Color := Color;
      SymbolsLB.Color := Color;
      AddKWMemo.Color := Color;
      AddSMemo.Color := Color;

      KeyWordsCG.ForegroundIndex := ColorToIndex(LangWordsList.KeyWordsColor);
      SymbolsCG.ForegroundIndex := ColorToIndex(LangWordsList.SymbolsColor);
    end;

    SyntaxHighlightingOnCB.Checked := ColorCodingOn;
    RecNumSB.Max := max(LangWordsList.Count - 1, 0);

    if LangWordsList.CurIndex = -1 then
      NewBtn.Click
    else
      CurRec := LangWordsList.CurIndex;
  end;
end;

procedure TSyntaxHighlightingForm.KeyWordsLBKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_DELETE)
  and RemoveKWBtn.Enabled
  then RemoveKWBtn.Click;
end;

procedure TSyntaxHighlightingForm.SymbolsLBKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_DELETE)
  and RemoveSBtn.Enabled
  then RemoveSBtn.Click;
end;

procedure TSyntaxHighlightingForm.KeyWordsLBKeyUp(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  SetEnables;
end;

procedure TSyntaxHighlightingForm.KeyWordsLBMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  SetEnables;
end;

procedure TSyntaxHighlightingForm.SymbolsLBMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  SetEnables;
end;

procedure TSyntaxHighlightingForm.SymbolsLBKeyUp(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  SetEnables;
end;

procedure TSyntaxHighlightingForm.RemoveKWBtnClick(Sender: TObject);
var i : integer;
begin
  i := KeyWordsLB.Items.Count;
  while (i > 0) do begin
    dec (i);
    if KeyWordsLB.Selected [i] then
      KeyWordsLB.Items.Delete (i);
  end;
  Modified := true;
end;

procedure TSyntaxHighlightingForm.RemoveSBtnClick(Sender: TObject);
var i : integer;
begin
  i := SymbolsLB.Items.Count;
  while (i > 0) do begin
    dec (i);
    if SymbolsLB.Selected [i] then
      SymbolsLB.Items.Delete (i);
  end;
  Modified := true;
end;

procedure TSyntaxHighlightingForm.AddKWMemoKeyPress(Sender: TObject;
  var Key: Char);
begin
  if Key = #27 then CancelBtn.Click;

  if Key = #13 then
    if AddKWMemo.Text = '' then
      OKBtn.Click
    else if not IsIdenifier(AddKWMemo.Text) then begin
      ShowMsgDlg('"' + AddKWMemo.Text + '" is not a valid idenifier', mtWarning);
      AddKWMemo.SelectAll;
    end else if IndexOf(KeyWordsLB.Items, AddKWMemo.Text, CaseSensitiveKWCB.Checked) = -1 then begin
      KeyWordsLB.Items.Add(AddKWMemo.Text);
      Modified := true;
      AddKWMemo.Clear;
    end else begin
      ShowMsgDlg('"' + AddKWMemo.Text + '" is already a key word', mtWarning);
      AddKWMemo.SelectAll;
    end;

  if Key = #13 then Key := #0;//prevents crlf
end;

procedure TSyntaxHighlightingForm.RecNumMemoKeyPress(Sender: TObject;
  var Key: Char);
var i : integer;
begin
  if not (Key in ValidNums) then Key := #0;
  if (Key = #13) and (RecNumMemo.Text <> '') then begin
    Key := #0;//don't add crlf to RecNumMemo
    i := StrToInt(RecNumMemo.Text) - 1;
    if i < 0 then i := 0;
    if i > RecNumSB.Max then i := RecNumSB.Max;
    CurRec := i;
  end;
end;

procedure TSyntaxHighlightingForm.NewBtnClick(Sender: TObject);
begin
  RecNumSB.Max := MainForm.LangWordsList.Count;
  CurRec := RecNumSB.Max;
end;

procedure TSyntaxHighlightingForm.RecNumSBScroll(Sender: TObject;
  ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
  CurRec := ScrollPos;
end;

procedure TSyntaxHighlightingForm.SaveBtnClick(Sender: TObject);
var LangWordsPtr : TLangReservedWordsPtr;
begin
  assert((CurRec >= 0) and (CurRec <= RecNumSB.Max));

  with MainForm do begin
    if CurRec = LangWordsList.Count then begin
      new(LangWordsPtr);
      LangWordsPtr^.Create;
      LangWordsList.Add(LangWordsPtr);
    end;

    assert(CurRec < LangWordsList.Count);
    with LangWordsList[CurRec]^ do begin
      KeyWords.MatchCase := CaseSensitiveKWCB.Checked;
      KeyWords.Items.Assign(KeyWordsLB.Items);

      Symbols.MatchCase := CaseSensitiveSCB.Checked;
      Symbols.Items.Assign(SymbolsLB.Items);

      Extensions.CommaText := ExtensionsEdit.Text;
    end;
    Modified := false;
    if (LangWordsList.CurIndex = CurRec) or
    LangWordsList.UpdateCurItem(MainForm.FileName) then begin
      LangWordsList.CurIndex := CurRec;//to get side effect
      ColorCodeAll;
    end;
  end;
end;

procedure TSyntaxHighlightingForm.ExtensionsEditChange(Sender: TObject);
begin
  Modified := true;
end;

procedure TSyntaxHighlightingForm.CaseSensitiveKWCBClick(Sender: TObject);
begin
  Modified := true;
end;

procedure TSyntaxHighlightingForm.NextBtnClick(Sender: TObject);
begin
  CurRec := CurRec + 1;
end;

procedure TSyntaxHighlightingForm.PrevBtnClick(Sender: TObject);
begin
  CurRec := CurRec - 1;
end;

procedure TSyntaxHighlightingForm.ReloadBtnClick(Sender: TObject);
begin
  if MessageDlg('Reload Current Record?', mtConfirmation, [mbYes, mbNo], 0) = mrNo then exit;
  Modified := false;
  CurRec := CurRec;
end;

procedure TSyntaxHighlightingForm.AddSMemoKeyPress(Sender: TObject;
  var Key: Char);
begin
  if Key = #27 then CancelBtn.Click;

  if Key = #13 then
    if AddSMemo.Text = '' then
      OKBtn.Click
    else if not IsValidSymbol(AddSMemo.Text) then begin
      ShowMsgDlg('"' + AddSMemo.Text + '" is not a valid symbol', mtWarning);
      AddSMemo.SelectAll;
    end else if IndexOf(SymbolsLB.Items, AddSMemo.Text, CaseSensitiveSCB.Checked) = -1 then begin
      SymbolsLB.Items.Add(AddSMemo.Text);
      Modified := true;
      AddSMemo.Clear;
    end else begin
      ShowMsgDlg('"' + AddSMemo.Text + '" is already a symbol', mtWarning);
      AddSMemo.SelectAll;
    end;

  if Key = #13 then Key := #0;//prevents crlf
end;

procedure TSyntaxHighlightingForm.CaseSensitiveSCBClick(Sender: TObject);
begin
  Modified := true;
end;

procedure TSyntaxHighlightingForm.OKBtnClick(Sender: TObject);
begin
  with MainForm do begin
    ColorCodingOn := SyntaxHighlightingOnCB.Checked;
    with LangWordsList do begin
      KeyWordsColor := KeyWordsCG.ForegroundColor;
      SymbolsColor := SymbolsCG.ForegroundColor;
    end;
    if CCModified then ColorCodeAll;
  end;
end;

procedure TSyntaxHighlightingForm.SyntaxHighlightingOnCBClick(
  Sender: TObject);
begin
  CCModified := true;
end;

procedure TSyntaxHighlightingForm.DeleteBtnClick(Sender: TObject);
var LangWordsPtr : TLangReservedWordsPtr;
begin
  if MessageDlg('Delete Current Record?', mtConfirmation, [mbYes, mbNo], 0) = mrNo then exit;

  with MainForm do begin
    assert((CurRec >= 0) and (CurRec < LangWordsList.Count));
    LangWordsPtr := LangWordsList[CurRec];
    LangWordsList.Delete(CurRec);
  end;
  assert(LangWordsPtr <> nil);
  LangWordsPtr^.Free;
  if RecNumSB.Max = RecNumSB.Min then
    NewBtn.Click
  else begin
    RecNumSB.Max := RecNumSB.Max - 1;
    CurRec := min(CurRec, RecNumSB.Max);
  end;
end;

procedure TSyntaxHighlightingForm.FormKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if Key = VK_F1 then HelpBtn.Click;
end;

procedure TSyntaxHighlightingForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if Modified and (PromptToSave = spNo) and
    (MessageDlg('Do you still want to close this dialog box?', mtConfirmation, [mbYes, mbNo], 0) = mrNo) then
    Action := caNone;
end;

end.
