unit ViewUnit;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, FileCtrl,
  CE_Unit, ReuUnit, ExtCtrls;

type
  TViewForm = class(TForm)
    FileEdit: TEdit;
    Label1: TLabel;
    ListBox1: TListBox;
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    ShowPathsCB: TCheckBox;
    FilterCB: TFilterComboBox;
    Label2: TLabel;
    Bevel1: TBevel;
    SortCB: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure ShowPathsCBClick(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure ListBox1Click(Sender: TObject);
    procedure FileEditChange(Sender: TObject);
    procedure FileEditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ListBox1DblClick(Sender: TObject);
    procedure ListBox1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FilterCBChange(Sender: TObject);
    procedure FileEditKeyPress(Sender: TObject; var Key: Char);
    procedure FileEditEnter(Sender: TObject);
    procedure SortCBClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ListBox1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure ListBox1Enter(Sender: TObject);
  private
    { Private declarations }
    procedure FillFilter;
    function AddToFilter (fName : TFileName) : boolean;
    procedure LoadToList;
    function SPName (i : byte) : TFileName;
  public
    { Public declarations }
  end;

var
  ViewForm: TViewForm;

implementation

uses MainUnit;

{$R *.DFM}

procedure TViewForm.FillFilter;
var i : byte;
begin
  for i := 0 to ListBox1.Items.Count - 1 do
    AddToFilter (ListBox1.Items[i]);
end;

function TViewForm.AddToFilter (fName : TFileName) : boolean;
begin
  fName := '*' + ExtractFileExt (fName);
  if length (fName) <= 2 then begin
    result := false;
    exit;
  end;

  fName := '|' + fName + '|' + fName;

  if posi (fName, FilterCB.Filter) = 0 then begin
    FilterCB.Filter := FilterCB.Filter + fName;
    result := true;
  end else result := false;
end;

procedure TViewForm.LoadToList;
var
  i : byte;
  fName : TFileName;
begin
  ListBox1.Clear;

  with MainForm do begin
    if FilesOpen <= 1 then exit;
    for i := 1 to FilesOpen - 1 do begin
      fName := SPName (i);
      if ListBox1.Items.IndexOf (fName) < 0 then
        ListBox1.Items.Add (fName);
    end;//for...
  end;//with...
end;

function TViewForm.SPName (i : byte) : TFileName;
var
  i2 : byte;
  fName : TFileName;
begin
  if ShowPathsCB.Checked then
    result := MainForm.Files[i]^.FileName
  else begin
    fName := ExtractFileName (MainForm.Files[i]^.FileName);
    {Last file is FilesOpen - 1 which 1stly
     doens't need to be checked and 2ndly
     would case an error in the for loop}
    if i < MainForm.FilesOpen - 1 then
      {i is current file, so i + 1 to
       FilesOpen - 1 is the rest of the files}
      for i2 := i + 1 to MainForm.FilesOpen - 1 do
        if StrCmpi (fName, ExtractFileName (MainForm.Files[i2]^.FileName)) then begin
          result := MainForm.Files[i]^.FileName;
          exit;
        end;

    if ListBox1.Items.Count > 0 then
      for i2 := 0 to ListBox1.Items.Count - 1 do
        if StrCmpi (fName, ExtractFileName (ListBox1.Items[i2])) then begin
          result := MainForm.Files[i]^.FileName;
          exit;
        end;

    result := ExtractFileName (MainForm.Files[i]^.FileName);
  end;
end;

procedure TViewForm.FormShow(Sender: TObject);
begin
  ShowPathsCB.Checked := MainForm.AnyFileNameOpenedTwice;
  FileEdit.Clear;
  ShowPathsCBClick(Sender);
  FileEdit.SetFocus;
  FileEditEnter(Sender);
  FilterCB.Filter := '*.*|*.*';
  FillFilter;
end;

procedure TViewForm.ShowPathsCBClick(Sender: TObject);
begin
  LoadToList;
  if ShowPathsCB.Checked then
    FileEdit.Text := MainForm.GetCommonPath + FileEdit.Text
  else
    FileEdit.Text := ExtractFileName(FileEdit.Text);
end;

procedure TViewForm.OKBtnClick(Sender: TObject);
var i : byte;
begin
  with ListBox1 do
    if ItemIndex >= 0 then
      FileEdit.Text := Items[ItemIndex]
    else if Items.Count = 1 then
      FileEdit.Text := Items[0];

  with MainForm do
    for i := 1 to FilesOpen - 1 do
      if ExtractFilePath (FileEdit.Text) <> '' then begin
        if StrCmpi (FileEdit.Text, Files[i]^.FileName) then begin
          ClickTabFile (OKBtn, i);
          exit;
        end;
      end else
        if StrCmpi (FileEdit.Text, ExtractFileName (Files[i]^.FileName)) then begin
          ClickTabFile (OKBtn, i);
          exit;
        end;

{   replace w/ above on 4-16-2000;
    it's here in case the Tabs idea doesn't work out

    if ViewMnu.Count > DefViewNum then
      for i := DefViewNum - 1 to ViewMnu.Count - 1 do
        if ExtractFilePath (FileEdit.Text) <> '' then begin
          if StrCmpi (FileEdit.Text, CapToStr (ViewMnu.Items[i].Caption)) then begin
            ViewMnu.Items[i].Click;
            exit;
          end;
        end else
          if StrCmpi (FileEdit.Text, ExtractFileName (CapToStr (ViewMnu.Items[i].Caption))) then begin
            ViewMnu.Items[i].Click;
            exit;
          end;
}
  if FileEdit.Text = '' then
    ShowMsgDlg ('No file was selected', mtInformation)
  else
    ShowMsgDlg ('File "' + FileEdit.Text + '" is not currently open or is the current file', mtInformation);

  {alredy exited if the file was copied or
   moved}
  ViewForm.ModalResult := mrNone;
end;

procedure TViewForm.ListBox1Click(Sender: TObject);
begin
  if ListBox1.ItemIndex >= 0 then
    FileEdit.Text := ListBox1.Items[ListBox1.ItemIndex];
end;

procedure TViewForm.FileEditChange(Sender: TObject);
var i : byte;
begin
  if (ListBox1.Items.Count = 0)
  or (FileEdit.GetTextLen = 0) then exit;

  for i := 0 to ListBox1.Items.Count - 1 do
    if StrCmpi (FileEdit.Text, copy (ListBox1.Items[i], 1, FileEdit.GetTextLen)) then begin
      ListBox1.ItemIndex := i;
      exit;
    end;

  ListBox1.ItemIndex := -1;
end;

procedure TViewForm.FileEditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  {This procedure moves the selected text when
   up, down, left, right, home, or end is
   pressed in FileEdit; there is similiar
   procedure for ListBox1KeyDown}

  with ListBox1 do begin
    if Shift = [ssCtrl] then
      if Key = VK_HOME then begin
        FileEdit.Text := Items[0];
        FileEdit.SelStart := FileEdit.GetTextLen;
        Key := 0;
      end else if Key = VK_END then begin
        FileEdit.Text := Items[Items.Count - 1];
        FileEdit.SelStart := FileEdit.GetTextLen;
        Key := 0;
      end;

    if Shift <> [] then exit;

    if Key = VK_DOWN then begin
      // if at bottom then move to top
      if ItemIndex = Items.Count - 1 then
        FileEdit.Text := Items[0]
      else // otherwise move down
        FileEdit.Text := Items[ItemIndex + 1];

      Key := 0;//keep cursor from moving right
      FileEdit.SelStart := FileEdit.GetTextLen;
    end else if Key = VK_UP then begin
      // if at top then move to bottom
      if ItemIndex <= 0 then
        FileEdit.Text := Items[Items.Count - 1]
      else // otherwise move up
        FileEdit.Text := Items[ItemIndex - 1];

      Key := 0;
      FileEdit.SelStart := FileEdit.GetTextLen;
    end;
  end; // with...
end;

procedure TViewForm.ListBox1DblClick(Sender: TObject);
begin
  OKBtn.Click;
end;

procedure TViewForm.ListBox1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  {This procedure moves the selected text  in ListBox1 when
   up or down is pressed and the selected text is at the
   top or bottom of ListBox1 respectively;
   there is similiar procedure for FileEditKeyDown}

  if Shift <> [] then exit;

  with ListBox1 do
    if (ItemIndex = 0) and (Key = VK_UP) then begin
      {makes selection go to the bottom if up
       arrow is pressed at top or Home key is
       pressed}
      ItemIndex := Items.Count - 1;
      Key := 0;//don't want to call FileEditChange
    end else if (ItemIndex = Items.Count - 1)
      and (Key = VK_DOWN) then begin
      {makes selection go to top if down arrow
       is pressed at bottom or End key is
       pressed}
      ItemIndex := 0;
      Key := 0;//don't want to call FileEditChange
    end;
end;

procedure TViewForm.FilterCBChange(Sender: TObject);
var i : byte;
begin
  LoadToList;

  if StrCmpi (FilterCB.Mask, '*.*') then exit;
  i := 0;

  while (i < ListBox1.Items.Count) do begin
    if not StrCmpi (ExtractFileExt (ListBox1.Items[i]), ExtractFileExt (FilterCB.Mask)) then
      ListBox1.Items.Delete (i)
    else
      inc (i);
  end;
end;

procedure TViewForm.FileEditKeyPress(Sender: TObject; var Key: Char);
begin
  {file FileEdit contains the name of a file in ListBox1
   and FileEdit's cursor is at 0, then the code below
   deletes it if the user try typing again}
  with ListBox1 do
    if (ItemIndex >= 0) and (ItemIndex < Items.Count)
    and (FileEdit.SelStart = 0) then
      if StrCmpi (FileEdit.Text, Items[ItemIndex]) then
        FileEdit.SelectAll;
end;

procedure TViewForm.FileEditEnter(Sender: TObject);
begin
  FileEdit.SelStart := FileEdit.GetTextLen;
end;

procedure TViewForm.SortCBClick(Sender: TObject);
begin
  ListBox1.Sorted := SortCB.Checked;
end;

procedure TViewForm.FormCreate(Sender: TObject);
begin
  SortCB.Checked := MainForm.SortViewFiles;
  SortCB.OnClick(Sender);
end;

procedure TViewForm.ListBox1MouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  ListBoxPosToHint(ListBox1, Application, X, Y);
end;

procedure TViewForm.ListBox1Enter(Sender: TObject);
begin
  ListBox1.Tag := -1;//used as last item index for ListBox1
end;

end.

